#include "BrowserApp.h"
#include "Handler/RenderProcessHandler.h"
#include "SiteKioskSchemeHandlerFactory.h"
#include "../Views/ViewManager.h"
#include "../Utils/CommandLine.h"
#include "../Utils/StringHelper.h"

#include <string>

#include "BrowserClient.h"
#include "include/cef_browser.h"
#include "include/cef_command_line.h"
#include "include/wrapper/cef_helpers.h"

CefRefPtr<BrowserClient> BrowserApp::client = nullptr;
BrowserApp* BrowserApp::instance = nullptr;

BrowserApp::BrowserApp()
	: renderProcessHandler(new RenderProcessHandler())
	, instanceId(GetCurrentProcessId())
{
	instance = this;
}


BrowserApp::~BrowserApp()
{
	client = nullptr;
	instance = nullptr;
}


void BrowserApp::OnBeforeCommandLineProcessing(const CefString& process_type, CefRefPtr<CefCommandLine> command_line)
{
	const auto& commandLine = CommandLine::GetInstance();

	command_line->AppendSwitch("disable-javascript-close-windows");
	command_line->AppendSwitch("force-renderer-accessibility");
	command_line->AppendSwitchWithValue("high-dpi-support", "1");
	command_line->AppendSwitchWithValue("force-device-scale-factor", "1");

	CefCommandLine::SwitchMap switches;
	command_line->GetSwitches(switches);
	command_line->Reset();

	const auto& rawCommandLineArgs = commandLine.GetRawArguments();
	for (const auto& arg : rawCommandLineArgs)
	{
		if (!StringHelper::StartsWith<wchar_t>(arg, L"cef-"))
			continue;

		auto cefArg = arg;

		cefArg = cefArg.erase(0, 4);
		auto add = !StringHelper::StartsWith<wchar_t>(cefArg, L"!");
		if (!add)
			cefArg = cefArg.erase(0, 1);

		auto cefArgElements = StringHelper::Split(cefArg, L'=', 2);

		for (auto iter = switches.begin(); iter != switches.end();)
		{
			auto key = std::wstring(iter->first);
			if (key == cefArgElements[0])
				switches.erase(iter++);
			else
				iter++;
		}

		if (add)
		{
			if (cefArgElements.size() == 1)
				switches.insert(std::make_pair(CefString(cefArgElements[0]), CefString()));
			else
				switches.insert(std::make_pair(CefString(cefArgElements[0]), CefString(cefArgElements[1])));
		}
	}
	for (const auto& pair : switches)
	{
		auto key = pair.first;
		auto value = pair.second;
		if (value.c_str() == nullptr)
			command_line->AppendSwitch(key);
		else
			command_line->AppendSwitchWithValue(key, value);
	}
}


void BrowserApp::OnContextInitialized()
{
	CEF_REQUIRE_UI_THREAD();

	CefRefPtr<CefCookieManager> manager = CefCookieManager::GetGlobalManager(nullptr);
	std::vector<CefString> cookieable_schemes;
	cookieable_schemes.push_back("http");
	cookieable_schemes.push_back("https");
	cookieable_schemes.push_back("ws");
	cookieable_schemes.push_back("wss");

	manager->SetSupportedSchemes(cookieable_schemes, nullptr);


	client = new BrowserClient();
	client->GetRPCProcessor().GetAppCommands().OnAppReady();
}


void BrowserApp::Run()
{
	ManualResetEvent& exitEvent = BrowserClient::GetInstance().GetShouldQuitEvent();
	ManualResetEvent& viewsEvent = BrowserClient::GetInstance().GetViewManager().GetViewExistingEvent();
	HANDLE waitEvents[2] = { exitEvent, viewsEvent };
	auto result = WaitForMultipleObjects(2, waitEvents, false, INFINITE);
	if (result == WAIT_OBJECT_0 + 1)
		CefRunMessageLoop();
}


CefRefPtr<CefRenderProcessHandler> BrowserApp::GetRenderProcessHandler()
{
	return renderProcessHandler;
}


void BrowserApp::OnRenderProcessThreadCreated(CefRefPtr<CefListValue> extra_info)
{
	extra_info->SetInt(0, GetCurrentProcessId());
}


void BrowserApp::OnRegisterCustomSchemes(CefRefPtr<CefSchemeRegistrar> registrar)
{
	registrar->AddCustomScheme("sk", false, false, false);
}


void BrowserApp::SetInstanceId(int instanceId)
{
	this->instanceId = instanceId;
}


int BrowserApp::GetInstanceId()
{
	return instanceId;
}
