#pragma once

#include "include/cef_client.h"
#include "../Views/ViewManager.h"
#include "../Messages/WebAppRPCProcessor.h"
#include "../Thread/ManualResetEvent.h"
#include "../Http/HttpRequestProvider.h"

#include <list>
#include <memory>
#include <condition_variable>

class DisplayHandler;
class LifeSpanHandler;
class LoadHandler;
class ContextMenuHandler;

class BrowserClient : 
	public CefClient
{
public:
	friend class LifeSpanHandler;

	BrowserClient();
	~BrowserClient();

	// Provide access to the single global instance of this object.
	static BrowserClient& GetInstance();
	static CefRefPtr<BrowserClient> GetRefPtrInstance();

	// CefClient methods:
	virtual CefRefPtr<CefDialogHandler> GetDialogHandler() OVERRIDE;
	virtual CefRefPtr<CefDisplayHandler> GetDisplayHandler() OVERRIDE;
	virtual CefRefPtr<CefLifeSpanHandler> GetLifeSpanHandler() OVERRIDE;
	virtual CefRefPtr<CefLoadHandler> GetLoadHandler() OVERRIDE;
	virtual CefRefPtr<CefContextMenuHandler> GetContextMenuHandler() OVERRIDE;
	virtual CefRefPtr<CefKeyboardHandler> GetKeyboardHandler() OVERRIDE;
	virtual CefRefPtr<CefRequestHandler> GetRequestHandler() OVERRIDE;
	virtual CefRefPtr<CefDownloadHandler> GetDownloadHandler() OVERRIDE;
	virtual CefRefPtr<CefRenderHandler> GetRenderHandler() OVERRIDE;
	virtual CefRefPtr<CefJSDialogHandler> GetJSDialogHandler() OVERRIDE;
	virtual CefRefPtr<CefDragHandler> GetDragHandler() OVERRIDE;
	virtual CefRefPtr<CefFocusHandler> GetFocusHandler() OVERRIDE;

	bool IsClosing() const { return isClosing; }

	ViewManager& GetViewManager();
	WebAppRPCProcessor& GetRPCProcessor();
	HttpRequestProvider& GetHttpRequestProvider();

	void Quit();
	void SetLanguage(const std::wstring& languageCode);
	std::wstring GetLanguage() const;
	ManualResetEvent& GetShouldQuitEvent();

private:
	bool isClosing;

	std::unique_ptr<WebAppRPCProcessor> rpcProcessor;
	std::unique_ptr<ViewManager> viewManager;
	std::unique_ptr<HttpRequestProvider> httpRequestProvider;

	CefRefPtr<CefDialogHandler> dialogHandler;
	CefRefPtr<CefDisplayHandler> displayHandler;
	CefRefPtr<CefLifeSpanHandler> lifeSpanHandler;
	CefRefPtr<CefLoadHandler> loadHandler;
	CefRefPtr<CefContextMenuHandler> contextMenuHandler;
	CefRefPtr<CefKeyboardHandler> keyboardHandler;
	CefRefPtr<CefRequestHandler> requestHandler;
	CefRefPtr<CefDownloadHandler> downloadHandler;
	CefRefPtr<CefRenderHandler> renderHandler;
	CefRefPtr<CefJSDialogHandler> jsDialogHandler;
	CefRefPtr<CefDragHandler> dragHandler;
	CefRefPtr<CefFocusHandler> focusHandler;

	ManualResetEvent shouldQuit;

	std::wstring languageCode;

	static BrowserClient* instance;

	IMPLEMENT_REFCOUNTING(BrowserClient);
};
