#include "ContextMenuHandler.h"
#include "../BrowserClient.h"
#include "../../Views/View.h"
#include "../../Views/ViewManager.h"
#include "../../Views/Components/InteractionComponent.h"
#include "include/wrapper/cef_helpers.h"

ContextMenuHandler::ContextMenuHandler()
{
}


ContextMenuHandler::~ContextMenuHandler()
{
}


void ContextMenuHandler::OnBeforeContextMenu(CefRefPtr<CefBrowser> browser,
	CefRefPtr<CefFrame> frame,
	CefRefPtr<CefContextMenuParams> params,
	CefRefPtr<CefMenuModel> model) {
	CEF_REQUIRE_UI_THREAD();

	auto view = BrowserClient::GetInstance().GetViewManager().GetView(browser);
	if ((view != nullptr && !view->Interaction.GetAllowContextMenu()) && !params->IsEditable())
		model->Clear();
	else if (!params->IsEditable())
	{
		auto appCommands = BrowserClient::GetInstance().GetRPCProcessor().GetAppCommands();
		model->Clear();
		
		model->AddItem(MENU_ID_BACK, appCommands.GetString(L"ContextMenu.Back"));
		model->SetEnabled(MENU_ID_BACK, browser->CanGoBack());

		model->AddItem(MENU_ID_FORWARD, appCommands.GetString(L"ContextMenu.Forward"));
		model->SetEnabled(MENU_ID_FORWARD, browser->CanGoForward());

		model->AddItem(MENU_ID_RELOAD, appCommands.GetString(L"ContextMenu.Reload"));

		if (view->Interaction.GetAllowPrinting())
			model->AddItem(MENU_ID_PRINT, appCommands.GetString(L"ContextMenu.Print"));
	}
	else
	{
		auto appCommands = BrowserClient::GetInstance().GetRPCProcessor().GetAppCommands();

		auto count = model->GetCount();
		for (int i = 0; i < count; ++i)
		{
			auto type = model->GetTypeAt(i);
			if (type == MENUITEMTYPE_SEPARATOR)
				continue;
			if (type != MENUITEMTYPE_COMMAND)
			{
				model->RemoveAt(i);
				continue;
			}
			auto commandID = model->GetCommandIdAt(i);
			switch (commandID)
			{
				case MENU_ID_UNDO:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.Undo"));
					break;
				case MENU_ID_REDO:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.Redo"));
					break;
				case MENU_ID_CUT:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.Cut"));
					break;
				case MENU_ID_COPY:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.Copy"));
					break;
				case MENU_ID_PASTE:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.Paste"));
					break;
				case MENU_ID_DELETE:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.Delete"));
					break;
				case MENU_ID_SELECT_ALL:
					model->SetLabelAt(i, appCommands.GetString(L"ContextMenu.SelectAll"));
					break;
				default:
					model->RemoveAt(i);
			}
		}
	}
}


bool ContextMenuHandler::OnContextMenuCommand(CefRefPtr<CefBrowser> browser,
	CefRefPtr<CefFrame> frame,
	CefRefPtr<CefContextMenuParams> params,
	int command_id,
	EventFlags event_flags) {
	CEF_REQUIRE_UI_THREAD();

	return false;
}