#include "DialogHandler.h"
#include "../BrowserClient.h"
#include "../../Views/View.h"
#include "../../Views/Components/RequestComponent.h"
#include "../../Views/ViewManager.h"
#include "../../Views/Components/EventComponent.h"

DialogHandler::DialogHandler()
{
}


DialogHandler::~DialogHandler()
{
}


bool DialogHandler::OnFileDialog(CefRefPtr<CefBrowser> browser, FileDialogMode mode, const CefString& title, const CefString& default_file_path, const std::vector<CefString>& accept_filters, int selected_accept_filter, CefRefPtr<CefFileDialogCallback> callback)
{
	auto view = BrowserClient::GetInstance().GetViewManager().GetView(browser);
	if (view == nullptr)
		return false;

	callback->AddRef();
	std::vector<std::wstring> acceptFilters;
	for (auto& acceptFilter : accept_filters)
		acceptFilters.push_back(acceptFilter);

	view->Requests.DisplayFileDialog(mode, title, default_file_path, acceptFilters, selected_accept_filter, [callback](const DisplayFileDialogResult& result) -> void
	{
		if (!result.Success)
		{
			callback->Cancel();
		}
		else
		{
			std::vector<CefString> files;
			for (auto& file : result.FilePaths)
				files.push_back(file);
			callback->Continue(result.SelectedAcceptFilter, files);
		}
		callback->Release();
	});

	return true;
}

