#pragma once

#include <json/json-forwards.h>
#include <atomic>
#include <mutex>
#include <memory>
#include "include\cef_resource_handler.h"
#include "include/cef_urlrequest.h"
#include "../Buffer.h"

class HttpRequest;

enum RequestFilterMode
{
	NoRequestFilter,
	FilterHTMLContent,
	FilterAllTextRequests
};

class SiteCoachResourceHandler :
	public CefResourceHandler
{
public:
	SiteCoachResourceHandler(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, CefRefPtr<CefRequest> request, RequestFilterMode requestFilterMode);
	~SiteCoachResourceHandler();

	virtual bool ProcessRequest(CefRefPtr<CefRequest> request, CefRefPtr<CefCallback> callback);
	virtual void GetResponseHeaders(CefRefPtr<CefResponse> response, int64& response_length, CefString& redirectUrl);
	virtual bool ReadResponse(void* data_out, int bytes_to_read, int& bytes_read, CefRefPtr<CefCallback> callback);
	virtual void Cancel();
	virtual bool CanGetCookie(const CefCookie& cookie);

private:
	bool MustScanRequest();
	bool MustCancelRequest();
	bool MustCancelRequestByUrl();
	bool MustCancelRequestByFilesUploaded();
	void FinishCallback(CefRefPtr<CefCallback>& callback, bool cancel);

private:
	std::recursive_mutex mutex;
	int viewId;
	CefRefPtr<CefBrowser> browser;
	CefRefPtr<CefFrame> frame;
	CefRefPtr<CefRequest> request;

	int pos;
	std::shared_ptr<HttpRequest> httpRequest;
	CefRefPtr<CefCallback> headerCallback;
	CefRefPtr<CefCallback> responseCallback;

	void* responseDataOut;
	RequestFilterMode requestFilterMode;

	IMPLEMENT_REFCOUNTING(SiteCoachResourceHandler);
};
