#include "SiteKioskResourceHandler.h"

#include "../BrowserClient.h"
#include "../../Views/ViewManager.h"
#include "../../Views/View.h"
#include "../../Utils/UriHelper.h"
#include "../../Utils/Uri.h"
#include "../../Utils/Profiler.h"
#include "../../Utils/StringHelper.h"
#include "../../Utils/CommandLine.h"
#include "../../Messages/RPCProcessor.h"

#include <json/json.h>
#include <algorithm>
#include <string>
#include <iostream>
#include <fstream>
#include <Shlobj.h>


SiteKioskResourceHandler::SiteKioskResourceHandler(CefRefPtr<CefFrame> frame)
	: responseData(nullptr)
	, responseDataLength(0)
	, pos(0)
	, frame(frame)
{
	viewId = BrowserClient::GetInstance().GetViewManager().GetView(frame->GetBrowser())->GetId();

	LPWSTR path = nullptr;
	auto hr = SHGetKnownFolderPath(FOLDERID_Public, KF_FLAG_DEFAULT, NULL, &path);
	if (SUCCEEDED(hr))
	{
		fileBaseDir = path;
		fileBaseDir += L"\\SiteKiosk\\data";
	}
}


SiteKioskResourceHandler::~SiteKioskResourceHandler()
{
	delete[] responseData;
}


bool SiteKioskResourceHandler::ProcessRequest(CefRefPtr<CefRequest> request, CefRefPtr<CefCallback> callback)
{
	auto uri = Uri::Parse(request->GetURL());

	std::transform(uri.Path.begin(), uri.Path.end(), uri.Path.begin(), ::tolower);

	std::ifstream file(fileBaseDir + uri.Path, std::ios::binary | std::ios::ate);
	if (file.is_open())
	{
		file.seekg(0, std::ios::end);

		responseDataLength = (size_t)file.tellg();

		responseData = new char[responseDataLength];
		file.seekg(0, std::ios::beg);
		file.read(responseData, responseDataLength);

		callback->Continue();
		return true;
	}
	else
	{
		//callback->Cancel();
		return false;
	}
}


void SiteKioskResourceHandler::GetResponseHeaders(CefRefPtr<CefResponse> response, int64& responseLength, CefString& redirectUrl)
{
	responseLength = responseData != nullptr ? responseDataLength : 0;
	redirectUrl = "";
}


bool SiteKioskResourceHandler::ReadResponse(void* dataOut,
	int bytesToRead,
	int& bytesRead,
	CefRefPtr<CefCallback> callback)
{
	if (bytesToRead == 0 || pos >= responseDataLength)
	{
		bytesRead = 0;
		return false;
	}
	else
	{
		memcpy(dataOut, &responseData[pos], bytesToRead);
		pos += bytesToRead;
		bytesRead = bytesToRead;
		return true;
	}
}


void SiteKioskResourceHandler::Cancel()
{
	// nop
}
