#include "SiteKioskObjectModel.h"
#include "../../../Messages/BrowserRPCProcessor.h"
#include "../../../Utils/StringHelper.h"

#include <iostream>
#include <fstream>
#include <json/json.h>


SiteKioskObjectModel::SiteKioskObjectModel()
{
}


SiteKioskObjectModel::~SiteKioskObjectModel()
{
}


bool SiteKioskObjectModel::CheckObjectModelAccess(CefRefPtr<CefV8Context> context)
{
	auto url = context->GetFrame()->GetURL().ToWString();

	std::unique_lock<std::recursive_mutex> lock(mutex);

	auto findIter = urlsWithObjectModelAccess.find(url);
	bool found = findIter != urlsWithObjectModelAccess.end();

	if (!found)
	{
		auto result = BrowserRPCProcessor::Get(context->GetBrowser())->GetCommands().MustApplyObjectModel(url);
		urlsWithObjectModelAccess[url] = result;
		return result;
	}

	return findIter->second;
}

bool SiteKioskObjectModel::Execute(const CefString& name, CefRefPtr<CefV8Value> object, const CefV8ValueList& arguments, CefRefPtr<CefV8Value>& retval, CefString& exception)
{
	auto context = CefV8Context::GetCurrentContext();

	if (!CheckObjectModelAccess(context))
	{
		exception = L"object model access denied.";
		OutputDebugString(L"SKNG: object model access denied.\n");
		return false;
	}

	if (name == L"invokeHostRpc" && arguments.size() == 1 && arguments[0]->IsString())
	{
		auto rpcProcessor = BrowserRPCProcessor::Get(context->GetBrowser());
		auto requestJson = (std::wstring)arguments[0]->GetStringValue();
		auto rpcResult = rpcProcessor->GetCommands().ExecuteRPC(requestJson);
		retval = CefV8Value::CreateString(rpcResult);

		return true;
	}
	else if (name == L"getHasSiteKiosk")
	{
		retval = CefV8Value::CreateBool(true);
		return true;
	}
	else if (name == L"getSiteKioskObjectModelCode")
	{
		std::unique_lock<std::recursive_mutex> lock(mutex);
		if (siteKioskScriptFileContent.empty())
		{
			auto rpcProcessor = BrowserRPCProcessor::Get(context->GetBrowser());
			auto assetsFilePath = rpcProcessor->GetCommands().GetAssetsPath();

			std::ifstream file(assetsFilePath + L"/sitekiosk/sitekiosk.min.js", std::ios::in | std::ios::binary);
			if (file)
			{
				std::string content;
				file.seekg(0, std::ios::end);
				content.resize((size_t)file.tellg());
				file.seekg(0, std::ios::beg);
				file.read(&content[0], content.size());
				file.close();

				siteKioskScriptFileContent = StringHelper::ToWString(content);
			}
		}
		retval = CefV8Value::CreateString(siteKioskScriptFileContent);
		return true;
	}
	return false;
}
