#pragma once

#include <cstring>
#include <exception>

template <typename T, bool SmartMemoryAllocation = true>
class Buffer
{
public:
	Buffer(T* baseData, size_t size)
		: Buffer()
	{
		Set(baseData, size);
	}

	Buffer()
		: size{ 0 }
		, allocated{ 0 }
		, data{ nullptr }
	{
	}

	~Buffer()
	{
		delete[] data;
	}

	T* Get() const
	{
		return data;
	}

	size_t Size() const
	{
		return size;
	}

	void Clear(bool freeMemory)
	{
		if (freeMemory)
		{
			delete[] data;
			data = nullptr;

			allocated = 0;
		}
		size = 0;
	}

	void Set(T* buffer, size_t bufferSize)
	{
		delete[] data;
		allocated = size = bufferSize;
		data = new T[size];
		std::memcpy(data, buffer, size);
	}

	void Append(T* buffer, size_t bufferSize)
	{
		auto neededSize = size + bufferSize;
		if (neededSize > allocated)
		{
			if (SmartMemoryAllocation)
				neededSize += bufferSize;

			auto newData = new T[neededSize];
			if (data != nullptr)
				std::memcpy(newData, data, size);
			delete[] data;
			data = newData;
			allocated = neededSize;
		}
		std::memcpy(&data[size], buffer, bufferSize);
		size += bufferSize;
	}

	size_t Consume(T* dest, size_t bytesToConsume)
	{
		bytesToConsume = min(bytesToConsume, size);
		std::memcpy(dest, data, bytesToConsume);
		if (bytesToConsume < size)
			std::memcpy(data, &data[bytesToConsume], size - bytesToConsume);
		size -= bytesToConsume;
		return bytesToConsume;
	}

	size_t Consume(Buffer<T>& dest, size_t bytesToConsume)
	{
		bytesToConsume = min(bytesToConsume, size);
		buffer.Append(data, bytesToConsume);
		if (bytesToConsume < size)
			std::memcpy(data, &data[bytesToConsume], size - bytesToConsume);
		size -= bytesToConsume;
		return bytesToConsume;
	}

private:
	T* data;
	size_t size;
	size_t allocated;
};
