#include "CABundleGenerator.h"

#include <iostream>
#include <fstream>
#include <vector>
#include <Windows.h>
#include <wincrypt.h> 

#include <openssl/x509.h>
#include <openssl/pem.h>

CABundleGenerator::CABundleGenerator(const std::string& caBundlePathDestination)
	: caBundlePathDestination(caBundlePathDestination)
{
}


CABundleGenerator::~CABundleGenerator()
{
}


bool CABundleGenerator::CABundleExists() const
{
	std::ifstream caBundleFile(caBundlePathDestination.c_str());
	return caBundleFile ? true : false;
}


void CABundleGenerator::Generate()
{
	if (CABundleExists())
		return;

	BIO* bio = BIO_new_file(caBundlePathDestination.c_str(), "w");
	if (bio == nullptr)
		return;

	WriteCertificates(bio, L"CA");
	WriteCertificates(bio, L"MY");
	WriteCertificates(bio, L"ROOT");

	std::ifstream input("cacert.pem", std::ios::binary);
	auto buffer = std::vector<char>((
		std::istreambuf_iterator<char>(input)),
		(std::istreambuf_iterator<char>()));
	BIO_write(bio, buffer.data(), buffer.size());

	BIO_free(bio);
}


void CABundleGenerator::WriteCertificates(BIO* bio, const std::wstring& subsystemProtocol)
{
	auto storeHandle = CertOpenSystemStore(NULL, subsystemProtocol.c_str());
	if (storeHandle == nullptr)
		return;

	PCCERT_CONTEXT certContext = nullptr;
	while (certContext = CertEnumCertificatesInStore(storeHandle, certContext))
	{
		if ((certContext->dwCertEncodingType & X509_ASN_ENCODING) != 0)
			WriteX509(certContext->pbCertEncoded, certContext->cbCertEncoded, bio);
	}

	CertCloseStore(storeHandle, 0);
}



void CABundleGenerator::WriteX509(const unsigned char* input, int length, BIO* bio)
{
	auto x509 = X509_new();
	if (x509 == nullptr)
		return;

	d2i_X509(&x509, &input, length);
	PEM_write_bio_X509(bio, x509);

	X509_free(x509);
}
