#pragma once

#include "HttpRequest.h"
#include "../Thread/ThreadPool.h"
#include "include/cef_client.h"
#include <memory>
#include <map>
#include <vector>
#include <mutex>
#include <queue>

class Cancelable;

struct StoredCurlHandle
{
	CURL* Handle;
	unsigned int StoredAtTime;
};

struct PendingCertificateRequests
{
	struct PendingCertificateRequest
	{
		PendingCertificateRequest(CertificateErrorRequest& request)
			: Request(request)
		{
		}

		CertificateErrorRequest& Request;
	};

	std::vector<PendingCertificateRequest> Requests;
	std::shared_ptr<Cancelable> Cancelable;

	void Cancel()
	{
		for (const auto& request : Requests)
			request.Request.Cancel();
	}

	void Continue()
	{
		for (const auto& request : Requests)
			request.Request.Continue();
	}
};

class HttpRequestProvider
{
	friend HttpRequest;

public:
	HttpRequestProvider();
	~HttpRequestProvider();

	std::shared_ptr<HttpRequest> Create(CefRefPtr<CefRequest> request);

private:
	bool CanNavigateToUnsafeUrl(const std::wstring& url);
	std::shared_ptr<PendingCertificateRequests> ProvidePendingCertificateRequests(const std::wstring& url, bool& isNew);
	void MakeCertificateErrorRequest(std::shared_ptr<PendingCertificateRequests> pendingRequests, int errorCode, const std::wstring& url);

	CURL* GetCURLHandle(const std::wstring& host);
	void ReleaseCURLHandle(const std::wstring& host, CURL* handle);
	void CheckOrphanedCURLHandles();

	std::string GetCABundlePath() const;

private:
	int minTLSSubVersion;
	std::map<std::wstring, std::wstring> domainToAuthMap;
	std::unique_ptr<ThreadPool> threadPool;

	std::map<std::wstring, std::shared_ptr<PendingCertificateRequests>> pendingCertificateErrorRequests;
	std::vector<std::wstring> allowedCertificateErrorDomains;
	std::recursive_mutex mutex;

	std::map<std::wstring, std::vector<StoredCurlHandle>> hostToUnusedCurlHandles;
	std::string caBundlePath;
};

