//#include <vld.h>
#include <windows.h>

#include "Browser/BrowserApp.h"
#include "Browser/BrowserClient.h"
#include "Messages/RPCProcessor.h"
#include "Utils/CommandLine.h"
#include "Utils/StringHelper.h"
#include "Utils/Profiler.h"
#include "Thread/ManualResetEvent.h"
#include "Messages/MessageProcessor/NetPipeMessageProcessor.h"
#include "ExceptionDump.h"

#include "Shlobj.h"
#include <iostream>

#include <io.h>
#include <fcntl.h>

#include <curl/curl.h>
#include <fstream>


int APIENTRY wWinMain(HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	PWSTR     lpCmdLine,
	int       nCmdShow)
{
	PERFORMANCER;
	ExceptionDump::InitializeExceptionHandling();

	SetConsoleOutputCP(CP_UTF8);
	SetConsoleCP(CP_UTF8);
	_setmode(_fileno(stdout), _O_U8TEXT);
	_setmode(_fileno(stdin), _O_U8TEXT);

	curl_global_init(CURL_GLOBAL_ALL);

	{
		CefMainArgs mainArgs(hInstance);
		CefRefPtr<BrowserApp> app(new BrowserApp);

		int exit_code = CefExecuteProcess(mainArgs, app.get(), nullptr);
		if (exit_code >= 0) // The sub-process has completed so return here.
		{
			curl_global_cleanup();
			PERFORMANCER_DUMP;
			return exit_code;
		}

		CefSettings settings;

		auto cachePath = CommandLine::GetInstance().GetStringArgument(L"cachePath");
		CefString(&settings.cache_path).FromWString(cachePath);
		settings.log_severity = LOGSEVERITY_DISABLE;

		std::ifstream debugTxtFile(".\\debug.txt");
		if (debugTxtFile)
			settings.remote_debugging_port = 2012;
		else
			settings.remote_debugging_port = -1;
		
		settings.windowless_rendering_enabled = false;
		settings.no_sandbox = true;

		auto fullUserAgent = CommandLine::GetInstance().GetStringArgument(L"fullUserAgent");
		if (!fullUserAgent.empty())
			CefString(&settings.user_agent).FromWString(fullUserAgent);

		CefInitialize(mainArgs, settings, app.get(), nullptr);
		app->Run();
		CefShutdown();
	}
	curl_global_cleanup();
	PERFORMANCER_DUMP;
	return 0;
}
