#pragma once
//#define USE_SHAREDMEMORY_IPC

#ifndef USE_SHAREDMEMORY_IPC
#include "MessageProcessor/NetPipeMessageProcessor.h"
#else
#include "MessageProcessor/SharedMemoryMessageProcessor.h"
#endif

#include "Commands/BrowserCommands.h"
#include "RPCProcessor.h"

#include "include/cef_app.h"

#include <map>
#include <memory>
#include <functional>

template<class T> class CreationCount
{
public:
	CreationCount(std::shared_ptr<T> data)
		: Data(data)
	{
	}
	
	void Set(std::shared_ptr<T> newData)
	{
		oldData = Data;
		Data = newData;
	}

	bool TryRemove(const std::function<void(std::shared_ptr<T>)> onRemove)
	{
		if (oldData != nullptr)
			oldData = nullptr;
		else
		{
			onRemove(Data);
			Data = nullptr;
		}

		return Data == nullptr;
	}

public:
	std::shared_ptr<T> Data;

private:
	std::shared_ptr<T> oldData;
};

class BrowserRPCProcessor
	: public RPCProcessor
{
public:
	static const std::wstring CommandPrefix;

public:
	BrowserRPCProcessor(CefRefPtr<CefBrowser> browser, bool isRenderProcess);
	~BrowserRPCProcessor();

	bool GetIsBrowser() const;

	void Shutdown();

	void RegisterCommand(const std::wstring& method, RPCFunction commandFunction);

	static void Add(CefRefPtr<CefBrowser> browser, bool isRenderProcess);
	static void Remove(CefRefPtr<CefBrowser> browser);
	static std::shared_ptr<BrowserRPCProcessor> Get(CefRefPtr<CefBrowser> browser);

	void CreateNewSendTarget(const std::wstring name);

public:
	BrowserCommands& GetCommands() { return commands; }
	std::wstring GetReceiverName() { return messageProcessor->GetReceiverName(); }

protected:
	virtual MessageProcessor& GetMessageProcessor() { return *messageProcessor; }

private:
	bool OnMessage(const std::wstring& message);

#ifndef USE_SHAREDMEMORY_IPC
	std::unique_ptr<NetPipeMessageProcessor> messageProcessor;
#else
	std::unique_ptr<SharedMemoryMessageProcessor> messageProcessor;
#endif
	CefRefPtr<CefBrowser> browser;
	std::vector<Handler> registeredCommands;
	static std::map<int, std::shared_ptr<CreationCount<BrowserRPCProcessor> > > browserToIPC;

	BrowserCommands commands;
	bool isRenderProcess;
};
