#pragma once

#include "../Thread/ManualResetEvent.h"
#include "include/cef_app.h"
#include <json/json-forwards.h>

class BrowserRPCProcessor;

class BrowserCommands
{
public:
	BrowserCommands(BrowserRPCProcessor &rpcProcessor, CefRefPtr<CefBrowser> browser);
	void Init();

	std::wstring ExecuteRPC(const std::wstring& rpc);
	void Ping();
	void Pong();
	void OpenDevTools();

	void OnRendererCreated(CefRefPtr<CefBrowser> browser);
	bool OnBeforeNavigation(const std::wstring& url, bool isMainFrame, int frameId);
	void OnContextCreated(int64 frameId);
	void OnContextReleased(int64 frameId);
	void OnEditableNodeFocusChanged(bool isEditable, int64 frameId);

	void OnViewIsReady();
	void ChangeCurrentRenderProcess(const std::wstring name);

	bool MustApplyObjectModel(const std::wstring& url);
	void Print();
	std::wstring GetAssetsPath() const;

private:
	bool OnBeforeNavigationHandler(const Json::Value& params, Json::Value& response);
	bool OnContextCreatedHandler(const Json::Value& params, Json::Value& response);
	bool OnContextReleasedHandler(const Json::Value& params, Json::Value& response);
	bool OnViewIsReadyHandler(const Json::Value& params, Json::Value& response);
	bool OnChangeCurrentRenderProcess(const Json::Value& params, Json::Value& response);
	bool OnRendererCreatedHandler(const Json::Value& params, Json::Value& response);
	bool OnExecuteRPCHandler(const Json::Value& params, Json::Value& response);
	bool OnPongHandler(const Json::Value& params, Json::Value& response);
	bool OnPingHandler(const Json::Value& params, Json::Value& response);
	bool OnOpenDevToolsHandler(const Json::Value& params, Json::Value& response);
	bool OnMustApplyObjectModelHandler(const Json::Value& params, Json::Value& response);
	bool OnEditableNodeFocusChangedHandler(const Json::Value& params, Json::Value& response);
	bool OnPrintHandler(const Json::Value& params, Json::Value& response);
	bool OnGetAssetsPath(const Json::Value& params, Json::Value& response);

	void PerformPing(CefRefPtr<CefBrowser> browser);
	
	BrowserRPCProcessor& rpcProcessor;
	CefRefPtr<CefBrowser> browser;
	bool viewIsReady;

private:
	std::wstring GenerateMessagePrefix() const;

	//IMPLEMENT_REFCOUNTING(BrowserCommands);
	// We bind the PerformPing method to post its call to the renderprocess main thread so we need to provide these methods
	// As we manage our lifetime by ourself we define the methods ourself.
public:
	void AddRef() const
	{
	}
	bool Release() const
	{
		return false;
	}
	bool HasOneRef() const 
	{
		return true;
	}
};

