#pragma once
#include "../../Utils/StringHelper.h"

#define BEGIN_RAW_COMMAND(rpcName, name, parameterCount) \
{ \
	rpcProcessor.RegisterViewCommand(rpcName, [&](const Json::Value& params, Json::Value& response, int viewId) -> bool { \
		if (params.size() != ##parameterCount##) \
			return false; \
		ViewManager& viewManager = BrowserClient::GetInstance().GetViewManager(); \
		auto view = viewManager.GetView(viewId); \
		if (view == nullptr) \
		{ \
			response["error"] = "View not found."; \
			return true; \
		} \

#define BEGIN_VOID_COMMAND(rpcName, name, parameterCount) BEGIN_RAW_COMMAND(rpcName, name, parameterCount) \
		response["result"] = Json::Value::null; \
		view->##name##(
#define BEGIN_COMMAND(rpcName, name, parameterCount) BEGIN_RAW_COMMAND(rpcName, name, parameterCount) \
		response["result"] = view->##name##(

#define BEGIN_WCOMMAND(rpcName, name, parameterCount) BEGIN_RAW_COMMAND(rpcName, name, parameterCount) \
		response["result"] = StringHelper::ToAString(view->##name##(

#define BEGIN_CAST_COMMAND(rpcName, name, parameterCount, type) BEGIN_RAW_COMMAND(rpcName, name, parameterCount) \
		response["result"] = (##type##)view->##name##(

#define INT_PARAMETER(index, defaultValue) params.get((Json::Value::ArrayIndex)##index##, ##defaultValue##).asInt()
#define FLOAT_PARAMETER(index, defaultValue) params.get((Json::Value::ArrayIndex)##index##, ##defaultValue##).asFloat()
#define BOOL_PARAMETER(index, defaultValue) params.get((Json::Value::ArrayIndex)##index##, ##defaultValue##).asBool()
#define STRING_PARAMETER(index, defaultValue) params.get((Json::Value::ArrayIndex)##index##, ##defaultValue##).asString()
#define WSTRING_PARAMETER(index, defaultValue) StringHelper::ToWString(params.get((Json::Value::ArrayIndex)##index##, ##defaultValue##).asString())

#define END_COMMAND \
		); \
		return true; \
	}); \
}

#define END_WCOMMAND \
		)); \
		return true; \
		}); \
}

#define VOID_COMMAND(rpcName, name) BEGIN_VOID_COMMAND(rpcName, name, 0) END_COMMAND
#define RESULT_COMMAND(rpcName, name) BEGIN_COMMAND(rpcName, name, 0) END_COMMAND
#define RESULT_CAST_COMMAND(rpcName, name, type) BEGIN_CAST_COMMAND(rpcName, name, 0, type) END_COMMAND
#define RESULT_WCOMMAND(rpcName, name) BEGIN_WCOMMAND(rpcName, name, 0) END_WCOMMAND

#define SET_VIEW_PROPERTY(name, method, type) \
{ \
	Json::Value val = params[0][name]; \
	if (!val.empty()) \
		view->##method##(val.as##type##()); \
}

#define SET_VIEW_INT_PROPERTY(name, method) SET_VIEW_PROPERTY(name, method, Int)
#define SET_VIEW_BOOL_PROPERTY(name, method) SET_VIEW_PROPERTY(name, method, Bool)
#define SET_VIEW_FLOAT_PROPERTY(name, method) SET_VIEW_PROPERTY(name, method, Float)
