#include "ViewCommands.h"
#include "ViewCommandMacros.h"
#include "../WebAppRPCProcessor.h"
#include "../MessageProcessor/MessageProcessor.h"
#include "../../Views/ViewManager.h"
#include "../../Views/View.h"
#include "../../Views/Components/LayoutComponent.h"
#include "../../Views/Components/DisplayComponent.h"
#include "../../Views/Components/InteractionComponent.h"
#include "../../Views/Components/RequestComponent.h"
#include "../../Views/Components/ErrorComponent.h"
#include "../../Browser/BrowserClient.h"
#include "../../Utils/StringHelper.h"

#include <climits>
#include <iostream>
#include <json/json.h>
#include <functional>


ViewCommands::ViewCommands(WebAppRPCProcessor& rpcProcessor)
	: rpcProcessor(rpcProcessor)
{
}


void ViewCommands::Init()
{
	VOID_COMMAND(L"reload", Interaction.Reload);
	VOID_COMMAND(L"stop", Interaction.Stop);
	VOID_COMMAND(L"forward", Interaction.Forward);
	VOID_COMMAND(L"back", Interaction.Back);
	VOID_COMMAND(L"close", Close);
	VOID_COMMAND(L"focus", Interaction.Focus);
	VOID_COMMAND(L"activate", Interaction.Activate);
	VOID_COMMAND(L"startFullscreen", Interaction.StartFullscreen);
	VOID_COMMAND(L"exitFullscreen", Interaction.ExitFullscreen);

	RESULT_WCOMMAND(L"url", Interaction.GetUrl);
	RESULT_COMMAND(L"canGoForward", Interaction.GetCanGoForward);
	RESULT_COMMAND(L"canGoBack", Interaction.GetCanGoBack);

	BEGIN_VOID_COMMAND(L"executeScript", Interaction.ExecuteScript, 1) WSTRING_PARAMETER(0, "") END_COMMAND;

	BEGIN_VOID_COMMAND(L"loadUrl", Interaction.LoadUrl, 2)
		WSTRING_PARAMETER(0, "about:blank"),
		INT_PARAMETER(1, -1)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"print", Interaction.Print, 1) WSTRING_PARAMETER(0, "SiteKioskNG") END_COMMAND;

	BEGIN_VOID_COMMAND(L"position", Layout.SetPosition, 3)
		INT_PARAMETER(0, 0),
		INT_PARAMETER(1, 0),
		BOOL_PARAMETER(2, false)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"size", Layout.SetSize, 2)
		INT_PARAMETER(0, 0),
		INT_PARAMETER(1, 0)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"minSize", Layout.SetMinSize, 2)
		INT_PARAMETER(0, 0),
		INT_PARAMETER(1, 0)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"maxSize", Layout.SetMaxSize, 2)
		INT_PARAMETER(0, 0),
		INT_PARAMETER(1, 0)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"positionAndSize", Layout.SetPositionAndSize, 5)
		INT_PARAMETER(0, 0),
		INT_PARAMETER(1, 0),
		INT_PARAMETER(2, 0),
		INT_PARAMETER(3, 0),
		BOOL_PARAMETER(4, false)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"setNonClientArea", Interaction.SetNonClientArea, 5)
		(NonClientAreaType)INT_PARAMETER(0, 0),
		INT_PARAMETER(1, 0),
		INT_PARAMETER(2, 0),
		INT_PARAMETER(3, 0),
		INT_PARAMETER(4, 0)
		END_COMMAND;

	BEGIN_VOID_COMMAND(L"width", Layout.SetWidth, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"width", Layout.GetWidth);

	BEGIN_VOID_COMMAND(L"height", Layout.SetHeight, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"height", Layout.GetHeight);

	BEGIN_VOID_COMMAND(L"left", Layout.SetLeft, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"left", Layout.GetLeft);

	BEGIN_VOID_COMMAND(L"top", Layout.SetTop, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"top", Layout.GetTop);

	BEGIN_VOID_COMMAND(L"relativeLeft", Layout.SetRelativeLeft, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"relativeLeft", Layout.GetRelativeLeft);

	BEGIN_VOID_COMMAND(L"relativeTop", Layout.SetRelativeTop, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"relativeTop", Layout.GetRelativeTop);

	BEGIN_VOID_COMMAND(L"parent", Layout.SetParent, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"parent", Layout.GetParent);

	BEGIN_VOID_COMMAND(L"hasBorder", Display.SetHasBorder, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"hasBorder", Display.GetHasBorder);

	BEGIN_VOID_COMMAND(L"allowInteraction", Interaction.SetAllowInteraction, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"allowInteraction", Interaction.GetAllowInteraction);

	BEGIN_VOID_COMMAND(L"zoom", Display.SetZoom, 1) FLOAT_PARAMETER(0, 1.0f) END_COMMAND;
	RESULT_COMMAND(L"zoom", Display.GetZoom);

	BEGIN_VOID_COMMAND(L"allowContextMenu", Interaction.SetAllowContextMenu, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"allowContextMenu", Interaction.GetAllowContextMenu);

	BEGIN_VOID_COMMAND(L"allowPrinting", Interaction.SetAllowPrinting, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"allowPrinting", Interaction.GetAllowPrinting);

	BEGIN_VOID_COMMAND(L"isVisible", Display.SetIsVisible, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"isVisible", Display.GetIsVisible);

	BEGIN_VOID_COMMAND(L"isCloseable", Interaction.SetIsCloseable, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"isCloseable", Interaction.GetIsCloseable);

	BEGIN_VOID_COMMAND(L"showIntaskbar", Display.SetShowInTaskbar, 1) BOOL_PARAMETER(0, true) END_COMMAND;
	RESULT_COMMAND(L"showIntaskbar", Display.GetShowInTaskbar);

	BEGIN_VOID_COMMAND(L"zIndex", Layout.SetZIndex, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"zIndex", Layout.GetZIndex);

	BEGIN_VOID_COMMAND(L"popupHandlingMode", Interaction.SetPopupHandling, 1) (PopupHandlingMode)params.get((Json::Value::ArrayIndex)0, 0).asInt() END_COMMAND;
	RESULT_CAST_COMMAND(L"popupHandlingMode", Interaction.GetPopupHandling, int);

	BEGIN_VOID_COMMAND(L"title", Display.SetTitle, 1) WSTRING_PARAMETER(0, L"") END_COMMAND;
	RESULT_WCOMMAND(L"title", Display.GetTitle);

	BEGIN_VOID_COMMAND(L"contentFilterLevel", Requests.SetContentFilterLevel, 1) INT_PARAMETER(0, 0) END_COMMAND;
	RESULT_COMMAND(L"contentFilterLevel", Requests.GetContentFilterLevel);

	rpcProcessor.RegisterViewCommand(L"updateViewProperties", std::bind(&ViewCommands::UpdateViewPropertiesHandler, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
	rpcProcessor.RegisterViewCommand(L"initialize", std::bind(&ViewCommands::InitializeHandler, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
	rpcProcessor.RegisterViewCommand(L"displayError", std::bind(&ViewCommands::DisplayErrorHandler, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
	rpcProcessor.RegisterNonBlockingViewCommand(L"closeForced", std::bind(&ViewCommands::CloseForcedHandler, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3, std::placeholders::_4));
	rpcProcessor.RegisterViewCommand(L"minSize", std::bind(&ViewCommands::GetMinSizeHandler, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
	rpcProcessor.RegisterViewCommand(L"maxSize", std::bind(&ViewCommands::GetMaxSizeHandler, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
}


std::wstring ViewCommands::GetString(int viewId, const std::wstring& stringId)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = viewId;
	params[1] = StringHelper::ToAString(stringId);
	auto response = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"getString")
		.UseParameters(params)
		.RunAndWait();
	
	return StringHelper::ToWString(response["result"].asString());
}


std::wstring ViewCommands::GenerateViewMessagePrefix(int viewId)
{
	PERFORMANCER;

	std::wstringstream stringStream;
	stringStream << WebAppRPCProcessor::ViewMessagePrefix;
	stringStream << viewId;
	stringStream << ":";

	return stringStream.str();
}


void ViewCommands::CancelRequest(int viewId, int requestId)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = requestId;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"cancelRequest")
		.UseParameters(params)
		.RunAndWait();
}


void ViewCommands::OnLoadStart(int viewId, CefRefPtr<CefFrame> frame)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(frame->GetURL());
	params[1] = frame->IsMain();

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"loadStart")
		.UseParameters(params)
		.Run();
}


void ViewCommands::OnLoadEnd(int viewId, CefRefPtr<CefFrame> frame, int httpStatusCode)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(frame->GetURL().c_str());
	params[1] = frame->IsMain();
	params[2] = frame->GetIdentifier();
	params[3] = httpStatusCode;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"loadEnd")
		.UseParameters(params)
		.Run();
}


void ViewCommands::OnLoadError(int viewId, CefRefPtr<CefFrame> frame, int errorCode, const std::wstring& errorText, const std::wstring& failedUrl)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = errorCode;
	params[1] = frame->IsMain();
	params[2] = StringHelper::ToAString(errorText);
	params[3] = StringHelper::ToAString(failedUrl);

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"loadError")
		.UseParameters(params)
		.Run();
}


void ViewCommands::OnLoadingStateChange(int viewId, bool isLoading, bool canGoBack, bool canGoForward)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = isLoading;
	params[1] = canGoBack;
	params[2] = canGoForward;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"loadingStateChange")
		.UseParameters(params)
		.Run();
}


void ViewCommands::OnTitleChanged(int viewId, const std::wstring& title)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(title);

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"titleChanged")
		.UseParameters(params)
		.Run();
}


void ViewCommands::OnFaviconURLChange(int viewId, const std::vector<std::wstring>& iconUrls)
{
	PERFORMANCER;

	Json::Value params;
	for (unsigned int i = 0; i < iconUrls.size(); ++i)
		params[i] = StringHelper::ToAString(iconUrls[i]);

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"faviconURLChange")
		.UseParameters(params)
		.Run();
}


void ViewCommands::OnFullscreenChanged(int viewId, bool isFullscreen)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = isFullscreen;
	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"fullscreenChanged")
		.UseParameters(params)
		.Run();
}


bool ViewCommands::OnCheckMustCancelRequest(int viewId, CefRefPtr<CefFrame> frame, const std::wstring& url, const std::wstring& content, CheckMustCancelRequestFlags checkFlags, HANDLE exitHandle)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(url);
	params[1] = frame->IsMain();
	params[2] = frame->GetIdentifier();
	params[3] = StringHelper::ToAString(content);
	params[4] = (int)checkFlags;

	auto response = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"checkMustCancelRequest")
		.UseParameters(params)
		.RunAndWait(exitHandle);

	return response["result"].asBool();
}


bool ViewCommands::OnFileUploadRequest(int viewId, CefRefPtr<CefFrame> frame, const std::wstring& url, const std::vector<std::wstring>& content, HANDLE exitHandle)
{
	PERFORMANCER;

	Json::Value files(Json::arrayValue);
	for (unsigned int i = 0; i < content.size(); ++i)
		files[i] = StringHelper::ToAString(content[i]);

	Json::Value params;
	params[0] = StringHelper::ToAString(url);
	params[1] = frame->IsMain();
	params[2] = frame->GetIdentifier();
	params[3] = files;

	auto response = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"fileUploadRequest")
		.UseParameters(params)
		.RunAndWait(exitHandle);

	return response["result"].asBool();
}


bool ViewCommands::OnBeforeNavigation(int viewId, const std::wstring& url, bool isMainFrame, int frameId, HANDLE exitHandle)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(url);
	params[1] = isMainFrame;
	params[2] = frameId;

	auto response = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"beforeNavigation")
		.UseParameters(params)
		.RunAndWait(exitHandle);

	return response["result"].asBool();
}


bool ViewCommands::OnBeforeBrowse(int viewId, const std::wstring& url, bool isMainFrame, bool isRedirect, int frameId)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(url);
	params[1] = isMainFrame;
	params[2] = isRedirect;
	params[3] = frameId;

	auto response = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"beforeBrowse")
		.UseParameters(params)
		.RunAndWait();

	return response["result"].asBool();
}

void ViewCommands::OnContextCreated(int viewId)
{
	PERFORMANCER;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"contextCreated")
		.RunAndWait();
}


void ViewCommands::OnContextReleased(int viewId, HANDLE exitHandle)
{
	PERFORMANCER;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"contextReleased")
		.RunAndWait(exitHandle);
}


void ViewCommands::OnClosed(int viewId)
{
	PERFORMANCER;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"closed")
		.Run();
}


void ViewCommands::OnGotFocus(int viewId)
{
	PERFORMANCER;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"gotFocus")
		.Run();
}


void ViewCommands::OnActivated(int viewId)
{
	PERFORMANCER;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"activated")
		.Run();
}


void ViewCommands::OnDeactivated(int viewId)
{
	PERFORMANCER;

	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"deactivated")
		.Run();
}


void ViewCommands::OnPopupViewCreated(int viewId, int popupViewId, int windowHandle)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = popupViewId;
	params[1] = windowHandle;
	rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"popupViewCreated")
		.UseParameters(params)
		.RunAndWait();
}


std::shared_ptr<Cancelable> ViewCommands::RequestAuthCredentials(int viewId, bool isProxy, const std::wstring& host, int port, const std::wstring& realm, const std::wstring& scheme, const std::function<void(const AuthenticationResult&)>& callback)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = isProxy;
	params[1] = StringHelper::ToAString(host);
	params[2] = port;
	params[3] = StringHelper::ToAString(realm);
	params[4] = StringHelper::ToAString(scheme);

	auto onResponse = [callback, viewId](const Json::Value& response) -> void
	{
		Json::Value result = response["result"];
		AuthenticationResult authResult;
		authResult.ShouldCancel = true;
		if (!result.empty())
		{
			authResult.Username = StringHelper::ToWString(result.get("username", "").asString());
			authResult.Password = StringHelper::ToWString(result.get("password", "").asString());
			authResult.ShouldCancel = authResult.Username.empty() || result.get("shouldCancel", false).asBool();
		}
		callback(authResult);
	};

	auto cancelable = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"requestAuthCredentials")
		.UseParameters(params)
		.RunAsync(onResponse);

	auto requestId = cancelable->GetId();
	cancelable->OnCancel([&, callback, viewId, requestId]() -> void {
		AuthenticationResult authResult;
		authResult.ShouldCancel = true;
		callback(authResult);
		CancelRequest(viewId, requestId);
	}, true);
	return cancelable;
}


std::shared_ptr<Cancelable> ViewCommands::RequestQuota(int viewId, const std::wstring& origin, int64 newSize, const std::function<void(bool)>& callback)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(origin);
	params[1] = newSize;

	auto onResponse = [callback, viewId](const Json::Value& response) -> void
	{
		callback(response["result"].asBool());
	};

	auto cancelable = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"requestQuota")
		.UseParameters(params)
		.RunAsync(onResponse);

	auto requestId = cancelable->GetId();
	cancelable->OnCancel([&, callback, viewId, requestId]() -> void {
		callback(false);
		CancelRequest(viewId, requestId);
	}, true);
	return cancelable;
}


std::shared_ptr<Cancelable> ViewCommands::RequestDownload(int viewId, const std::wstring& url, int64 totalBytes, const std::function<void(const DownloadRequestResult&)>& callback)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(url);
	params[1] = totalBytes;

	auto onResponse = [callback, viewId](const Json::Value& response) -> void
	{
		Json::Value result = response["result"];
		DownloadRequestResult downloadRequestResult;
		downloadRequestResult.ShouldCancel = true;
		if (!result.empty())
		{
			downloadRequestResult.Destination = StringHelper::ToWString(result.get("destination", "").asString());
			downloadRequestResult.ShouldCancel = downloadRequestResult.Destination.empty() || result.get("shouldCancel", false).asBool();
		}
		callback(downloadRequestResult);
	};

	auto cancelable = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"requestDownload")
		.UseParameters(params)
		.RunAsync(onResponse);

	auto requestId = cancelable->GetId();
	cancelable->OnCancel([&, callback, viewId, requestId]() -> void {
		DownloadRequestResult downloadRequestResult;
		downloadRequestResult.ShouldCancel = true;
		callback(downloadRequestResult);
		CancelRequest(viewId, requestId);
	}, true);
	return cancelable;
}


std::shared_ptr<Cancelable> ViewCommands::DisplayDialog(int viewId, const std::wstring& origin, const std::wstring& acceptLang, int dialogType, const std::wstring& message, const std::wstring& defaultPromptText, const std::function<void(const DisplayDialogResult&)>& callback)
{
	PERFORMANCER;

	Json::Value params;
	params[0] = StringHelper::ToAString(origin);
	params[1] = StringHelper::ToAString(acceptLang);
	params[2] = dialogType;
	params[3] = StringHelper::ToAString(message);
	params[4] = StringHelper::ToAString(defaultPromptText);

	auto onResponse = [callback, viewId](const Json::Value& response) -> void
	{
		Json::Value result = response["result"];
		DisplayDialogResult displayDialogResult;
		displayDialogResult.Success = result.get("success", false).asBool();
		displayDialogResult.UserInput = StringHelper::ToWString(result.get("userInput", "").asString());
		
		callback(displayDialogResult);
	};

	auto cancelable = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"displayDialog")
		.UseParameters(params)
		.RunAsync(onResponse);

	auto requestId = cancelable->GetId();
	cancelable->OnCancel([&, callback, viewId, requestId]() -> void {

		DisplayDialogResult displayDialogResult;
		displayDialogResult.Success = false;
		callback(displayDialogResult);
		CancelRequest(viewId, requestId);
	}, false);
	return cancelable;
}


std::shared_ptr<Cancelable> ViewCommands::DisplayFileDialog(int viewId, int mode, const std::wstring& title, const std::wstring& defaultFilePath, const std::vector<std::wstring>& acceptFilters, int selectedAcceptFilter, const std::function<void(const DisplayFileDialogResult&)>& callback)
{
	PERFORMANCER;

	Json::Value jsonAcceptFilters(Json::arrayValue);
	for (unsigned int i = 0; i < acceptFilters.size(); ++i)
		jsonAcceptFilters[i] = StringHelper::ToAString(acceptFilters[i]);

	Json::Value params;
	params[0] = mode;
	params[1] = StringHelper::ToAString(title);
	params[2] = StringHelper::ToAString(defaultFilePath);
	params[3] = jsonAcceptFilters;
	params[4] = selectedAcceptFilter;

	auto onResponse = [callback, viewId](const Json::Value& response) -> void
	{
		Json::Value result = response["result"];
		DisplayFileDialogResult displayFileDialogResult;
		displayFileDialogResult.Success = result.get("success", false).asBool();
		displayFileDialogResult.SelectedAcceptFilter = result.get("selectedAcceptFilter", 0).asInt();
		auto jsFilePaths = result.get("filePaths", Json::Value(Json::arrayValue));

		for (unsigned int i = 0; i < jsFilePaths.size(); ++i)
		{
			auto string = jsFilePaths.get(i, "").asString();
			displayFileDialogResult.FilePaths.push_back(StringHelper::ToWString(string));
		}

		callback(displayFileDialogResult);
	};

	auto cancelable = rpcProcessor.CreateRequest(GenerateViewMessagePrefix(viewId), L"displayFileDialog")
		.UseParameters(params)
		.RunAsync(onResponse);

	auto requestId = cancelable->GetId();
	cancelable->OnCancel([&, callback,  viewId, requestId]() -> void {
		DisplayFileDialogResult displayFileDialogResult;
		displayFileDialogResult.Success = false;
		callback(displayFileDialogResult);
		CancelRequest(viewId, requestId);
	}, true);
	return cancelable;
}


bool ViewCommands::UpdateViewPropertiesHandler(const Json::Value& params, Json::Value& response, int viewId)
{
	PERFORMANCER;

	auto& viewManager = BrowserClient::GetInstance().GetViewManager();
	auto view = viewManager.GetView(viewId);
	if (view == nullptr)
	{
		response["error"] = L"View not found.";
		return true;
	}

	SET_VIEW_INT_PROPERTY(	 "width",			 Layout.SetWidth);
	SET_VIEW_INT_PROPERTY(	 "height",			 Layout.SetHeight);
	SET_VIEW_INT_PROPERTY(	 "left",			 Layout.SetLeft);
	SET_VIEW_INT_PROPERTY(	 "top",				 Layout.SetTop);
	SET_VIEW_INT_PROPERTY(	 "relativeLeft",	 Layout.SetRelativeLeft);
	SET_VIEW_INT_PROPERTY(	 "relativeTop",		 Layout.SetRelativeTop);
	SET_VIEW_INT_PROPERTY(	 "parent",			 Layout.SetParent);
	SET_VIEW_INT_PROPERTY(	 "zIndex",			 Layout.SetZIndex);

	SET_VIEW_BOOL_PROPERTY(	 "hasBorder",		 Display.SetHasBorder);
	SET_VIEW_FLOAT_PROPERTY( "zoom",			 Display.SetZoom);
	SET_VIEW_BOOL_PROPERTY(	 "isVisible",		 Display.SetIsVisible);

	SET_VIEW_BOOL_PROPERTY("allowContextMenu",   Interaction.SetAllowContextMenu);
	SET_VIEW_BOOL_PROPERTY("allowInteraction",   Interaction.SetAllowInteraction);
	SET_VIEW_BOOL_PROPERTY("allowPrinting",		 Interaction.SetAllowPrinting);

	auto minWidth = params[0]["minWidth"];
	auto minHeight = params[0]["minHeight"];
	if (!minWidth.empty() && !minHeight.empty()) \
		view->Layout.SetMinSize(minWidth.asInt(), minHeight.asInt());

	auto maxWidth = params[0]["maxWidth"];
	auto maxHeight = params[0]["maxHeight"];
	if (!maxWidth.empty() && !maxHeight.empty()) \
		view->Layout.SetMaxSize(maxWidth.asInt(), maxHeight.asInt());

	return true;
}


bool ViewCommands::InitializeHandler(const Json::Value& params, Json::Value& response, int viewId)
{
	PERFORMANCER;

	auto& viewManager = BrowserClient::GetInstance().GetViewManager();
	auto view = viewManager.GetView(viewId);
	if (view == nullptr)
	{
		response["error"] = L"View not found.";
		return true;
	}

	auto options = params[0];

	CreateViewParams createViewParams;

	createViewParams.ZIndex = options.get("zIndex", 0).asInt();
	createViewParams.Left = options.get("left", 0).asInt();
	createViewParams.Top = options.get("top", 0).asInt();
	createViewParams.Width = options.get("width", 640).asInt();
	createViewParams.Height = options.get("height", 480).asInt();
	createViewParams.ShowBorders = options.get("showBorders", true).asBool();
	createViewParams.IsClosable = options.get("isClosable", true).asBool();
	createViewParams.StartUrl = StringHelper::ToWString(options.get("url", "about:blank").asString());
	createViewParams.ParentId = options.get("parentId", -1).asInt();
	createViewParams.AllowInteraction = options.get("allowInteraction", true).asBool();
	createViewParams.IsVisible = options.get("isVisible", true).asBool();
	createViewParams.ShowInTaskbar = options.get("showInTaskbar", true).asBool();
	createViewParams.ContentFilterLevel = options.get("contentFilterLevel", true).asInt();

	createViewParams.MinWidth = options.get("minWidth", 0).asInt();
	createViewParams.MinHeight = options.get("minHeight", 0).asInt();
	createViewParams.MaxWidth = options.get("maxWidth", INT_MAX).asInt();
	createViewParams.MaxHeight = options.get("maxHeight", INT_MAX).asInt();

	view->Init(createViewParams);

	response["result"] = (int)view->GetHandle();

	return true;
}


bool ViewCommands::DisplayErrorHandler(const Json::Value& params, Json::Value& response, int viewId)
{
	PERFORMANCER;

	auto& viewManager = BrowserClient::GetInstance().GetViewManager();
	auto view = viewManager.GetView(viewId);
	if (view == nullptr)
	{
		response["error"] = L"View not found.";
		return true;
	}

	auto failedUrl = StringHelper::ToWString(params[0].asString());
	auto errorCode = params[1].asInt();
	auto frameId = params.size() > 2 ? params[2].asInt() : -1;


	view->Error.DisplayError(failedUrl, errorCode, frameId);

	return true;
}


bool ViewCommands::CloseForcedHandler(const Json::Value& parameters, Json::Value response, std::function<void(const Json::Value&)> onFinished, int viewId)
{
	PERFORMANCER;

	auto& viewManager = BrowserClient::GetInstance().GetViewManager();
	viewManager.CloseView(viewId, [onFinished, response]()
	{
		onFinished(response);
	});

	return true;
}


bool ViewCommands::GetMinSizeHandler(const Json::Value& params, Json::Value& response, int viewId)
{
	if (params.size() != 0) 
		return false;

	auto& viewManager = BrowserClient::GetInstance().GetViewManager();
	auto view = viewManager.GetView(viewId);
	if (view == nullptr)
	{
		response["error"] = L"View not found.";
		return true;
	}

	auto minSize = view->Layout.GetMinSize();

	Json::Value result;
	result["width"] = minSize.cx;
	result["height"] = minSize.cy;

	response["result"] = result;

	return true;
}


bool ViewCommands::GetMaxSizeHandler(const Json::Value& params, Json::Value& response, int viewId)
{
	if (params.size() != 0)
		return false;

	auto& viewManager = BrowserClient::GetInstance().GetViewManager();
	auto view = viewManager.GetView(viewId);
	if (view == nullptr)
	{
		response["error"] = L"View not found.";
		return true;
	}

	auto maxSize = view->Layout.GetMaxSize();

	Json::Value result;
	result["width"] = maxSize.cx;
	result["height"] = maxSize.cy;

	response["result"] = result;

	return true;
}
