#pragma once

#include "../Views/View.h"
#include "../Cancelable.h"

#include "include/cef_app.h"

#include <json/json-forwards.h>
#include <string>
#include <vector>
#include <functional>
#include "../../Views/Components/EventComponent.h"

class WebAppRPCProcessor;
class ViewManager;
class RPCProcessor;
struct AuthenticationResult;
struct DownloadRequestResult;
struct DisplayDialogResult;
struct DisplayFileDialogResult;

class ViewCommands
{
public:
	ViewCommands(WebAppRPCProcessor &rpcProcessor);
	void Init();

	std::wstring GetString(int viewId, const std::wstring& stringId);

	void OnLoadStart(int viewId, CefRefPtr<CefFrame> frame);
	void OnLoadEnd(int viewId, CefRefPtr<CefFrame> frame, int httpStatusCode);
	void OnLoadError(int viewId, CefRefPtr<CefFrame> frame, int errorCode, const std::wstring& errorText, const std::wstring& failedUrl);
	void OnLoadingStateChange(int viewId, bool isLoading, bool canGoBack, bool canGoForward);

	void OnTitleChanged(int viewId, const std::wstring& title);
	void OnFaviconURLChange(int viewId, const std::vector<std::wstring>& iconUrls);

	void OnFullscreenChanged(int viewId, bool isFullscreen);

	void CancelRequest(int viewId, int requestId);

	bool OnCheckMustCancelRequest(int viewId, CefRefPtr<CefFrame> frame, const std::wstring& url, const std::wstring& content, CheckMustCancelRequestFlags checkFlags, HANDLE exitHandle);
	bool OnFileUploadRequest(int viewId, CefRefPtr<CefFrame> frame, const std::wstring& url, const std::vector<std::wstring>& content, HANDLE exitHandle);

	bool OnBeforeNavigation(int viewId, const std::wstring& url, bool isMainFrame, int frameId, HANDLE exitHandle);
	bool OnBeforeBrowse(int viewId, const std::wstring& url, bool isMainFrame, bool isRedirect, int frameId);
	void OnContextCreated(int viewId);
	void OnContextReleased(int viewId, HANDLE exitHandle);
	void OnClosed(int viewId);
	void OnGotFocus(int viewId);

	void OnActivated(int viewId);
	void OnDeactivated(int viewId);

	void OnPopupViewCreated(int viewId, int popupViewId, int windowHandle);

	std::shared_ptr<Cancelable> RequestAuthCredentials(int viewId, bool isProxy, const std::wstring& host, int port, const std::wstring& realm, const std::wstring& scheme, const std::function<void(const AuthenticationResult&)>& callback);
	std::shared_ptr<Cancelable> RequestQuota(int viewId, const std::wstring& origin, int64 newSize, const std::function<void(bool)>& callback);
	std::shared_ptr<Cancelable> RequestDownload(int viewId, const std::wstring& url, int64 totalBytes, const std::function<void(const DownloadRequestResult&)>& callback);
	std::shared_ptr<Cancelable> DisplayDialog(int viewId, const std::wstring& origin, const std::wstring& acceptLang, int dialogType, const std::wstring& message, const std::wstring& defaultPromptText, const std::function<void(const DisplayDialogResult&)>& callback);
	std::shared_ptr<Cancelable> DisplayFileDialog(int viewId, int mode, const std::wstring& title, const std::wstring& defaultFilePath, const std::vector<std::wstring>& acceptFilters, int selectedAcceptFilter, const std::function<void(const DisplayFileDialogResult&)>& callback);

private:
	bool UpdateViewPropertiesHandler(const Json::Value& params, Json::Value& response, int viewId);
	bool InitializeHandler(const Json::Value& params, Json::Value& response, int viewId);
	bool DisplayErrorHandler(const Json::Value& params, Json::Value& response, int viewId);
	bool CloseForcedHandler(const Json::Value& parameters, Json::Value response, std::function<void(const Json::Value&)> onFinished, int subTargetId);

	bool GetMinSizeHandler(const Json::Value& params, Json::Value& response, int viewId);
	bool GetMaxSizeHandler(const Json::Value& params, Json::Value& response, int viewId);

private:
	std::wstring GenerateViewMessagePrefix(int viewID);
	WebAppRPCProcessor &rpcProcessor;
};

