#pragma once

#include "../../Thread/ManualResetEvent.h"
#include "../../Utils/Profiler.h"
#include <string>
#include <map>
#include <functional>
#include <mutex>
#include <memory>
#include <atomic>


class MessageProcessor
{
	class Handler
	{
	public:
		Handler(const std::function<bool(const std::wstring&)>& method, bool oneTimeUseOnly)
			: Method(method)
			, OneTimeUseOnly(oneTimeUseOnly)
		{
			HasBeenUsed = false;
			runCount = 0;
		}

		bool TryExecute(const std::wstring& message)
		{
			runCount++;
			if (HasBeenUsed)
			{
				runCount--;
				return false;
			}

			PERFORMANCER;

			if (Method(message))
			{
				HasBeenUsed = HasBeenUsed || OneTimeUseOnly;
				runCount--;
				return true;
			}
			runCount--;
			return false;
		}

		void Abort()
		{
			PERFORMANCER;

			HasBeenUsed = true;
			while (runCount > 0)
				Sleep(1);
		}

		std::function<bool(const std::wstring&)> Method;
		bool OneTimeUseOnly;
		std::atomic_bool HasBeenUsed;
		int Id;

	private:
		std::atomic_int runCount;
	};

public:
	MessageProcessor(const std::wstring& sendTarget);
	virtual ~MessageProcessor();

	void RemoveMessageHandler(int handlerId);
	int AddMessageHandler(std::function<bool(const std::wstring&)> method, bool oneTimeUseOnly = false);
	int AddMessageHandler(const std::wstring& prefix, std::function<bool(const std::wstring&)> method, bool oneTimeUseOnly = false);

	virtual void Send(const std::wstring& message) = 0;
	virtual void Start() = 0;
	virtual void Shutdown();

	virtual void ChangeSendTarget(const std::wstring& newSendTarget);
	std::wstring GetSendTarget();

protected:
	virtual void OnMessageReceived(const std::wstring& message);

	std::wstring sendTarget;
	bool isShuttingDown;

	std::mutex mutex;
	std::mutex shutDownMutex;
	std::mutex processingMutex;
	std::map<int, std::shared_ptr<Handler>> messageHandlers;
	int nextMessageHandlerId;
};

