#include "StdInOutMessageProcessor.h"
#include "MessageProcessor.h"
#include "../WebAppRPCProcessor.h"
#include "../../Utils/ThreadHelper.h"
#include "../../Utils/Profiler.h"
#include "../../resource.h"
#include "../../Browser/BrowserApp.h"

#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

StdInOutMessageProcessor::StdInOutMessageProcessor()
	: MessageProcessor(L"StdInOut")
	, shouldStop(false)
{
	
}

StdInOutMessageProcessor::~StdInOutMessageProcessor()
{
	Shutdown();
	thread->join();
	threadPool = nullptr;
}


void StdInOutMessageProcessor::Start()
{
	thread = std::unique_ptr<std::thread>(new std::thread(&StdInOutMessageProcessor::ProcessMessagesAsync, this));

	threadPool = std::unique_ptr<ThreadPool>(new ThreadPool("StdInOut", 3, INFINITE, 10000));

	ThreadHelper::SetThreadName(*thread, "StdInOutMessageProcessor Receiver");
}


void StdInOutMessageProcessor::Shutdown()
{
	if (shouldStop)
		return;

	shouldStop = true;
	Send(WebAppRPCProcessor::AppCommandPrefix + L"{\"id\":0,\"method\":\"shutdown\"}");
	threadPool = nullptr;
}


void StdInOutMessageProcessor::ProcessMessagesAsync()
{
	std::wstring line;
	try
	{
		while (!shouldStop)
		{
			std::getline(std::wcin, line);
			if (!line.empty() && !shouldStop && threadPool)
			{
				threadPool->Dispatch([=]
				{
					OnMessageReceived(line);
				});
			}
		}
	}
	catch (...)
	{
		OutputDebugString(L"SKNG:CefApp: StdInOutMessageProcessor Thread has failed.\n");
	}
}


void StdInOutMessageProcessor::Send(const std::wstring& message)
{
	PERFORMANCER;

	std::unique_lock<std::mutex> lock(mutex);
	if (isShuttingDown)
		return;

	std::wcout << message << std::endl;
	if (!std::wcout.good())
	{
		std::wcout.clear();
		OutputDebugString(L"WCOUT is in a bad state.");
	}
}
