#pragma once

#include "PipeInstance.h"

#include <windows.h> 
#include <stdio.h>
#include <strsafe.h>
#include <string>
#include <thread>
#include <array>
#include <memory>

// We need more than one instance so that more than one entity can be connect to the same instance,
// for example: LoadURL may create a new RenderProcess, this process must be able to connect
// to the BrowserProcess so that the RenderProcess can tell the BrowserProcess to kick away the old
// Renderprocess.
#define SERVER_PIPE_INSTANCES 2

class NetPipeReader
{
public:
	NetPipeReader(const std::wstring& name, const std::function<void(const std::wstring&)>& onMessage);
	~NetPipeReader();

private:
	void ProcessMessagesAsync();
	void FreeResources();

private:
	std::array<PipeInstance, SERVER_PIPE_INSTANCES> pipes;
	std::array<ManualResetEvent, SERVER_PIPE_INSTANCES + 1> events; // first element is our exitEvent

	std::wstring pipeName;

	std::unique_ptr<std::thread> thread;

	ManualResetEvent& exitEvent;
};

