#pragma once

#include <windows.h> 
#include <stdio.h>
#include <strsafe.h>
#include <string>
#include <thread>
#include <memory>
#include <mutex>
#include <queue>
#include <functional>

#include "../../Thread/ManualResetEvent.h"
#include "../../Thread/AutoResetEvent.h"
#include "../../Utils/Handle.h"

class NetPipeWriter
{
public:
	NetPipeWriter(const std::wstring& name);
	NetPipeWriter(const std::wstring& name, const std::function<void(NetPipeWriter*)>& shutdownCallback);
	~NetPipeWriter();

	void SetShutdownCallback(const std::function<void(NetPipeWriter*)>& shutdownCallback);

	void Send(const std::wstring& message);
	std::wstring GetName() const { return name; }

private:
	void ProcessMessagesAsync();
	bool SendMessageOverPipe(const Handle& pipeInstance, const std::string& message);

	Handle EstablishConnection();

private:
	std::wstring pipeName;
	std::wstring name;
	std::unique_ptr<std::thread> thread;

	ManualResetEvent exitEvent;
	AutoResetEvent hasMessagesEvent;
	std::mutex mutex;
	std::queue<std::wstring> pendingMessages;

	std::function<void(NetPipeWriter*)> shutdownCallback;
};

