#pragma once

#include "../../Thread/ManualResetEvent.h"
#include "../../Utils/Handle.h"
#include <windows.h> 
#include <strsafe.h>
#include <string>
#include <array>
#include <memory>
#include <sstream>
#include <functional>


#define PIPE_TIMEOUT 5000
#define SERVER_PIPE_BUFSIZE 4096

enum PipeState
{
	Connecting,
	Reading,
	Writing
};


class PipeInstance
{
public:
	PipeInstance();
	~PipeInstance();
	void Close();
	void Connect(int pipeInstanceCount, const std::wstring& pipeName, const Handle& waitEvent, const std::function<void(const std::wstring&)>& onMessage);

	void ProcessMessages();

private:
	bool TryGetPendingIOResult();
	void ConnectToNewClient();
	void DisconnectAndReconnect();

private:
	OVERLAPPED overlapped;
	Handle pipeInstance;
	std::array<char, SERVER_PIPE_BUFSIZE> requestBuffer;
	DWORD readBytes;
	std::array<char, SERVER_PIPE_BUFSIZE> responseBuffer;
	DWORD bytesToWrite;
	PipeState state;
	bool hasPendingIO;

	int messageLength;
	int receivedMessageLength;
	std::stringstream receivedMessage;

	std::function<void(const std::wstring&)> onMessage;
};
