#include "RPCRequest.h"
#include "RPCProcessor.h"


///////////////////////////////////////////////////////////////////////////
RPCRequest::RPCRequest(RPCProcessor& owner, const std::wstring& prefix, const std::wstring& method, int requestId)
	: prefix(prefix)
	, method(method)
	, requestId(requestId)
	, waitForResponse(false)
	, waitForResponseHandle(nullptr)
	, owner(owner)
{

}


RPCRequest::RPCRequest(RPCProcessor& owner, const std::wstring& prefix, int requestId, const std::wstring& jsonString)
	: prefix(prefix)
	, requestId(requestId)
	, waitForResponse(false)
	, waitForResponseHandle(nullptr)
	, rawJsonRequest(jsonString)
	, owner(owner)
{

}


RPCRequest& RPCRequest::UseResponsePrefix(const std::wstring& prefix)
{
	this->responsePrefix = prefix;
	return *this;
}


RPCRequest& RPCRequest::UseParameters(const Json::Value& parameters)
{
	this->parameters = parameters;
	return *this;
}

std::shared_ptr<Cancelable>RPCRequest::RunAsync(const std::function<void(const Json::Value&)>& callback)
{
	responseCallback = callback;

	return owner.SendRequest(*this).GetCancelable();
}


Json::Value RPCRequest::RunAndWait(HANDLE abortHandle)
{
	waitForResponse = true;
	waitForResponseHandle = abortHandle;

	return owner.SendRequest(*this).GetJsonResponse();
}


void RPCRequest::Run()
{
	owner.SendRequest(*this);
}
