#pragma once

//#include "RPCProcessor.h"
#include "Cancelable.h"
#include <windows.h>
#include <string>
#include <json/json.h>
#include <memory>
#include <functional>

class RPCProcessor;

class RPCRequest
{
public:
	RPCRequest(RPCProcessor& owner, const std::wstring& prefix, const std::wstring& method, int requestId = -1);
	RPCRequest(RPCProcessor& owner, const std::wstring& prefix, int requestId, const std::wstring& jsonString);

	RPCRequest& UseResponsePrefix(const std::wstring& responsePrefix);
	RPCRequest& UseParameters(const Json::Value& parameters);

	std::shared_ptr<Cancelable> RunAsync(const std::function<void(const Json::Value&)>& callback);
	Json::Value RunAndWait(HANDLE abortHandle = nullptr);
	void Run();

protected:

	std::wstring rawJsonRequest;
	std::wstring prefix;
	std::wstring method;
	std::wstring responsePrefix;
	Json::Value parameters;
	bool waitForResponse;
	HANDLE waitForResponseHandle;
	int requestId;
	RPCProcessor& owner;

	std::function<void(const Json::Value&)> responseCallback;
};


class RPCRequestInternal : public RPCRequest
{
public:
	RPCRequestInternal(const RPCRequest& request) : RPCRequest(request) {  }

	bool HasResponseCallback() const { return (bool)responseCallback; }
	std::function<void(const Json::Value&)> GetResponseCallback() const { return responseCallback; }
	void RemoveResponseCallback() { responseCallback = nullptr; }
	std::wstring GetPrefix() const { return prefix; }
	std::wstring GetMethod() const { return method; }
	std::wstring GetResponsePrefix() const { return responsePrefix; }
	Json::Value GetParameters() const { return parameters; }
	bool IsWaitingForResponse() const { return waitForResponse; }
	HANDLE GetAbortHandle() const { return waitForResponseHandle; }
	int GetRequestId() const { return requestId; }
	std::wstring GetRawJsonRequest() const { return rawJsonRequest; }
	void SetRequestId(int id) { requestId = id; }
};