#pragma once

#include <string>
#include <vector>
#include <thread>
#include <memory>
#include <sstream>

#include "../../Thread/Mutex.h"
#include "../../Thread/ManualResetEvent.h"

#include "MemoryMappedFile.h"

class IPCReader
{
public:
	IPCReader(const std::wstring& name, const std::function<void(const std::wstring&)>& onMessage, size_t capacity = 1024 * 1024 * 2 /* 2 Megabyte */);
	IPCReader();
	~IPCReader();

private:
	void ProcessMessagesAsync();
	size_t Read();

private:
	std::wstring name;
	Mutex dataMutex;
	ManualResetEvent dataAvailableEvent;
	MemoryMappedFile memoryMappedFile;
	size_t capacity;

	std::unique_ptr<std::thread> thread;
	ManualResetEvent exitEvent;
	
	int messageLength;
	int receivedMessageLength;
	std::stringstream receivedMessage;
	std::function<void(const std::wstring&)> onMessage;

	char* readBuffer;
};

