#pragma once

#include <windows.h> 
#include <stdio.h>
#include <strsafe.h>
#include <string>
#include <thread>
#include <memory>
#include <mutex>
#include <queue>
#include <functional>

#include "../../Thread/Mutex.h"
#include "../../Thread/ManualResetEvent.h"
#include "../../Thread/AutoResetEvent.h"
#include "../../Utils/Handle.h"

#include "MemoryMappedFile.h"

class IPCWriter
{
public:
	static bool CanOpen(const std::wstring& name);
	static bool CanCreate(const std::wstring& name);
public:
	IPCWriter(const std::wstring& name, const std::function<void(IPCWriter*)>& shutdownCallback, size_t capacity = 1024 * 1024);
	~IPCWriter();
	bool TryConnect(const std::wstring& name);

	void Write(const char* buffer, size_t size);

	void SetShutdownCallback(const std::function<void(IPCWriter*)>& shutdownCallback);

	void Write(const std::wstring& message);
	std::wstring GetName() const { return name; }

private:
	void ProcessMessagesAsync();
	size_t WriteInternal(const char* buffer, size_t size, size_t offset);

private:
	size_t capacity;
	std::wstring name;
	Mutex dataMutex;
	ResetEventBase dataAvailableEvent;
	MemoryMappedFile memoryMappedFile;

	std::unique_ptr<std::thread> thread;

	ManualResetEvent exitEvent;
	AutoResetEvent hasMessagesEvent;
	std::mutex mutex;
	std::queue<std::wstring> pendingMessages;

	std::function<void(IPCWriter*)> shutdownCallback;
};

