#include "MemoryMappedFile.h"
#include <utility>

MemoryMappedFile::MemoryMappedFile()
	: handle(nullptr)
{
	
}


MemoryMappedFile::MemoryMappedFile(MemoryMappedFile&& other)
	: handle(std::move(other.handle))
{
	other.handle = nullptr;
}


MemoryMappedFile::MemoryMappedFile(const std::wstring& name)
{
	handle = ::OpenFileMapping(
		FILE_MAP_ALL_ACCESS,   // read/write access
		FALSE,                 // do not inherit the name
		name.c_str());               // name of mapping object
}


MemoryMappedFile::MemoryMappedFile(const std::wstring& name, size_t capacity)
{
	handle = ::CreateFileMapping(
		INVALID_HANDLE_VALUE,    // use paging file
		nullptr,                    // default security
		PAGE_READWRITE,          // read/write access
		0,                       // maximum object size (high-order DWORD)
		capacity,                // maximum object size (low-order DWORD)
		name.c_str());                 // name of mapping object
}


MemoryMappedFile::~MemoryMappedFile()
{
	if (handle != nullptr)
		::CloseHandle(handle);
}


MemoryMappedFile::ViewAccessor MemoryMappedFile::CreateViewAccessor()
{
	auto mappedView = (char*)::MapViewOfFile(handle,   // handle to map object
		FILE_MAP_ALL_ACCESS, // read/write permission
		0,
		0,
		0);

	auto lasterror = GetLastError();

	return MemoryMappedFile::ViewAccessor(mappedView);
}


MemoryMappedFile& MemoryMappedFile::operator= (MemoryMappedFile& other)
{
	std::swap(handle, other.handle);
	return *this;
}


////////////////////////////////////////////////////////////////////////
MemoryMappedFile::ViewAccessor::ViewAccessor(char* mappedView)
	: mappedView(mappedView)
{
}


MemoryMappedFile::ViewAccessor::ViewAccessor(MemoryMappedFile::ViewAccessor&& other)
	: mappedView(std::move(other.mappedView))
{
	other.mappedView = nullptr;
}


MemoryMappedFile::ViewAccessor::~ViewAccessor()
{
	if (mappedView != nullptr)
		::UnmapViewOfFile(mappedView);
}


MemoryMappedFile::ViewAccessor& MemoryMappedFile::ViewAccessor::operator = (MemoryMappedFile::ViewAccessor other)
{
	std::swap(mappedView, other.mappedView);
	return *this;
}
