#pragma once

#include "RPCProcessor.h"
#include "MessageProcessor/StdInOutMessageProcessor.h"
#include "Commands/AppCommands.h"
#include "Commands/ViewCommands.h"

class WebAppRPCProcessor :
	public RPCProcessor
{
public:
	// Commands prefixed by this prefix should execute a simple action
	static const std::wstring AppCommandPrefix;

	// A script evokes an objectModel call, we need to transfer this to the C# environment which will handle it
	// we also need to wait for a response to it which we will need to send back to the invoker
	static const std::wstring ObjectModelMessagePrefix;

	// Commands send from WebApp to CefApp to manage the views
	static const std::wstring ViewMessagePrefix;

public:
	WebAppRPCProcessor();
	~WebAppRPCProcessor();

	void RegisterNonBlockingAppCommand(const std::wstring& method, RPCFunctionNonBlocking commandFunction);
	void RegisterAppCommand(const std::wstring& method, RPCFunction commandFunction);
	void RegisterNonBlockingViewCommand(const std::wstring& method, RPCFunctionNonBlocking commandFunction);
	void RegisterViewCommand(const std::wstring& method, RPCFunction commandFunction);

	AppCommands& GetAppCommands() { return appCommands; }
	ViewCommands& GetViewCommands() { return viewCommands; }

	void Shutdown();

private:
	bool OnAppMessage(const std::wstring& message);
	bool OnViewMessage(const std::wstring& message);

protected:
	virtual MessageProcessor& GetMessageProcessor() { return *messageProcessor; }
	virtual std::wstring PrettifyMessagePrefix(const std::wstring& prefix);

private:
	std::unique_ptr<StdInOutMessageProcessor> messageProcessor;
	AppCommands appCommands;
	ViewCommands viewCommands;

	std::vector<Handler> registeredAppCommands;
	std::vector<Handler> registeredViewCommands;
};
