#include "ResetEventBase.h"
#include <sstream>
#include <string>


HANDLE ResetEventBase::CreateEvent(bool initialState, bool manualReset, const std::wstring& name)
{
	HANDLE handle = nullptr;
	do
	{
		handle = ::CreateEvent(nullptr, manualReset, initialState, name.empty() ? nullptr : name.c_str());
		if (handle == nullptr && GetLastError() != ERROR_ALREADY_EXISTS)
		{
			throw new std::exception("Unable to create Event.");
		}
	} while (handle == nullptr);

	return handle;
}


ResetEventBase::ResetEventBase()
{
}


ResetEventBase::ResetEventBase(bool initialState, bool manualReset, const std::wstring& name)
	: Handle(CreateEvent(initialState, manualReset, name))
{
}


ResetEventBase::ResetEventBase(const std::wstring& name)
	: Handle(::OpenEvent(EVENT_ALL_ACCESS | EVENT_MODIFY_STATE, false, name.c_str()))
{
}


ResetEventBase::ResetEventBase(ResetEventBase&& other)
	: Handle(std::move(other))
{
}


void ResetEventBase::Set()
{
	::SetEvent(GetHandle());
}


void ResetEventBase::Reset()
{
	::ResetEvent(GetHandle());
}


DWORD ResetEventBase::Wait(DWORD timeout) const
{
	return WaitForSingleObject(GetHandle(), timeout);
}
