#pragma once

#include "../Utils/Handle.h"
#include <array>
#include <algorithm>
#include "windows.h"

class ResetEventBase : public Handle
{
public:
	ResetEventBase();
	ResetEventBase(const std::wstring& name);
	ResetEventBase(bool initialState, bool manualReset, const std::wstring& name = L"");
	ResetEventBase(ResetEventBase&& other);

	void Set();
	void Reset();

	DWORD Wait(DWORD timeout = INFINITE) const;

protected:
	ResetEventBase(const ResetEventBase& other);

public: // static
	static HANDLE CreateEvent(bool initialState, bool manualReset, const std::wstring& name);

	template<class TEventBase, std::size_t N> static std::array<HANDLE, N> GenerateHandlesArray(const std::array<TEventBase, N>& events)
	{
		std::array<HANDLE, N> handles;
		for (std::size_t i = 0; i < N; ++i)
			handles[i] = events[i];
		return handles;
	}

	template<std::size_t N> static DWORD Wait(const std::array<HANDLE, N>& handles, bool waitAll = false, DWORD timeout = INFINITE)
	{
		return WaitForMultipleObjects(N, handles.data(), waitAll, INFINITE);
	}

	template<class TEventBase, std::size_t N> static DWORD Wait(const std::array<TEventBase, N>& events, bool waitAll = false, DWORD timeout = INFINITE)
	{
		auto handles = GenerateHandlesArray(events);
		return WaitForMultipleObjects(N, handles.data(), waitAll, timeout);
	}
};

