#pragma once

#include "ManualResetEvent.h"
#include "AutoResetEvent.h"
#include <vector>
#include <functional>
#include <mutex>
#include <thread>
#include <memory>
#include <queue>

class ThreadPool
{
public:
	ThreadPool(const std::string& name, unsigned int minThreadCount, unsigned int maxThreadCount = INFINITE, DWORD threadTimeOut = 1000);
	~ThreadPool();

	void Dispatch(const std::function<void()>& method);

private:
	class Thread
	{
		public:
			Thread(const std::string& name, ThreadPool& owner, bool isCoreThread);
			~Thread();
			bool IsDead() { return isDead; }
			bool IsIdle();
		
		private:
			void ProcessMessagesAsync();

		private:
			std::unique_ptr<std::thread> thread;
			ManualResetEvent isWorkingEvent;
			ThreadPool& owner;
			bool isDead;
			bool isCoreThread;
	};

	void CleanUp();
	void SpawnNewThread();

	std::function<void()> ConsumeEvent();
	bool AllThreadsOccupied();

private:
	std::vector<std::shared_ptr<Thread>> threads;
	std::queue<std::function<void()>> availableTasks;

	std::mutex mutex;
	ManualResetEvent exitEvent;
	ManualResetEvent tasksAvailableEvent;

	unsigned int minThreadCount;
	unsigned int maxThreadCount;
	DWORD threadTimeOut;
	std::string name;
};

