#include "CommandLine.h"
#include "StringHelper.h"
#include "windows.h"

CommandLine* CommandLine::instance;

CommandLine::CommandLine()
{
	int argCount;
	auto argList = CommandLineToArgvW(GetCommandLineW(), &argCount);
	if (argList == nullptr)
		return;

	for (auto count = 1; count < argCount; ++count)
		ParseCommandLineArgument(argList[count]);
}


void CommandLine::ParseCommandLineArgument(const std::wstring& arg)
{
	rawParameters.push_back(arg);
	auto args = StringHelper::Split<wchar_t>(arg, L'=', 2);
	if (args.size() == 1)
		args.push_back(L"");

	auto key   = args[0];
	auto value = args[1];
	std::transform(key.begin(), key.end(), key.begin(), ::tolower);

	parameters.insert(std::pair<std::basic_string<wchar_t>, std::basic_string<wchar_t> >(key, value));
}


const std::wstring* CommandLine::GetArgumentRaw(const std::wstring& arg) const
{
	auto key = arg;
	std::transform(key.begin(), key.end(), key.begin(), ::tolower);

	auto findResult = parameters.find(key);
	if (findResult == parameters.end())
		return nullptr;
	else
		return &findResult->second;
}

CommandLine& CommandLine::GetInstance()
{
	if (instance == nullptr)
		instance = new CommandLine();
	return *instance;
}


std::wstring CommandLine::GetStringArgument(const std::wstring& arg, const std::wstring &defaultValue) const
{
	auto value = GetArgumentRaw(arg);
	if (value == nullptr)
		return defaultValue;
	return *value;
}

bool CommandLine::GetBoolArgument(const std::wstring& arg, const bool &defaultValue) const
{
	auto value = GetArgumentRaw(arg);
	if (value == nullptr)
		return defaultValue;

	auto boolValue = *value;
	std::transform(boolValue.begin(), boolValue.end(), boolValue.begin(), ::tolower);
	return boolValue.empty() || boolValue == L"true";
}


int CommandLine::GetIntArgument(const std::wstring& arg, const int &defaultValue) const
{
	auto value = GetArgumentRaw(arg);
	if (value == nullptr)
		return defaultValue;

	return std::stoi(*value);
}


long CommandLine::GetLongArgument(const std::wstring& arg, const long &defaultValue) const
{
	auto value = GetArgumentRaw(arg);
	if (value == nullptr)
		return defaultValue;

	return std::stol(*value);
}


float CommandLine::GetFloatArgument(const std::wstring& arg, const float &defaultValue) const
{ const
	auto value = GetArgumentRaw(arg);
	if (value == nullptr)
		return defaultValue;

	return std::stof(*value);
}


const std::vector<std::wstring>& CommandLine::GetRawArguments() const
{
	return rawParameters;
}
