#include "Performancer.h"
#include "Windows.h"

namespace
{
	const long long g_Frequency = []() -> long long
	{
		LARGE_INTEGER frequency;
		QueryPerformanceFrequency(&frequency);
		return frequency.QuadPart;
	}();
}

HighResClock::time_point HighResClock::now()
{
	LARGE_INTEGER count;
	QueryPerformanceCounter(&count);
	return time_point(duration(count.QuadPart * static_cast<rep>(period::den) / g_Frequency));
}

#if PROFILE

#include <iostream>

Performancer Performancer::Instance;

int Performancer::RegisterMethod(const std::string& name, const std::string& fullName)
{
	data.push_back({ name, fullName });
	return data.size() - 1;
}

void Performancer::AddData(int index, long long duration)
{
	data[index].Add(duration);
}

struct my_numpunct : std::numpunct<char> {
	std::string do_grouping() const { return "\03"; }
};

void Performancer::Dump()
{
	auto sorted = data;
	std::sort(sorted.begin(), sorted.end(), [](const PerformancerData& lhs, const PerformancerData& rhs)
	{
		return lhs.Elapsed > rhs.Elapsed;
	});

	long long total = 0;
	long long average = 0;
	size_t maxNameLength = 0;
	for (const auto& data : sorted)
	{
		maxNameLength = data.Name.size() > maxNameLength ? data.Name.size() : maxNameLength;
		total += data.Elapsed;
		if (data.Calls > 0)
			average += data.Elapsed / data.Calls;
	}

	maxNameLength += 2;

	const int usWidth = 20;
	std::locale loc(std::cout.getloc(), new my_numpunct);
	std::stringstream ss;
	ss.imbue(loc);
	ss << "Performancer summary:\n";
	ss.width(10);
	ss << std::left << "Rank";
	ss.width(maxNameLength);
	ss << std::left << "Name";
	ss.width(usWidth);
	ss << std::left << "Calls";
	ss.width(usWidth);
	ss << std::left << "Min (us)";
	ss.width(usWidth);
	ss << std::left << "Max (us)";
	ss.width(usWidth);
	ss << std::left << "Average (us)";
	ss.width(usWidth);
	ss << std::left << "Average (%)";
	ss.width(usWidth);
	ss << std::left << "Total (us)";
	ss.width(usWidth);
	ss << std::left << "Total (%)";
	ss.width(usWidth);
	ss << "Full Name";
	ss << "\n";

	

	int i = 0;
	for (const auto& data : sorted)
	{
		i++;
		ss.width(10);
		ss << std::left << i;
		ss.width(maxNameLength);
		ss << std::left << data.Name;
		ss.width(usWidth);
		ss << std::left << data.Calls;
		ss.width(usWidth);
		ss << std::left << data.Min;
		ss.width(usWidth);
		ss << std::left << data.Max;
		ss.width(usWidth);
		ss << std::left << (data.Calls > 0 ? data.Elapsed / data.Calls : 0);
		ss.width(usWidth);
		ss << std::left << (data.Calls > 0 ? (int)(((data.Elapsed / data.Calls) / (double)average) * 100) : 0);
		ss.width(usWidth);
		ss << std::left << data.Elapsed;
		ss.width(usWidth);
		ss << std::left << (total > 0 ? (int)((data.Elapsed / (double)total) * 100) : 0);
		ss << data.FullName;
		ss << "\n";
	}
	ss << "\n";

	OutputDebugStringA(ss.str().c_str());
}


PerformancerEntry::PerformancerEntry(int index)
	: index{ index }
	, start{ HighResClock::now() }
{

}

PerformancerEntry::~PerformancerEntry()
{
	auto end = HighResClock::now();
	auto duration = std::chrono::duration_cast<std::chrono::microseconds>(end - start).count();
	Performancer::Instance.AddData(index, duration);
}

PerformancerData::PerformancerData(const std::string& name, const std::string& fullName)
	: Name(name)
	, FullName(fullName)
	, Calls{ 0 }
	, Elapsed{ 0 }
	, Max{ 0 }
	, Min{ 0 }
{
}

void PerformancerData::Add(long long duration)
{
	if (Calls == 0)
	{
		Min = duration;
		Max = duration;
	}
	else
	{
		Min = Min > duration ? duration : Min;
		Max = Max < duration ? duration : Max;
	}

	Elapsed += duration;
	Calls++;
}
#endif