#pragma once

#include <sstream>
#include <iostream>
#include <string>
#include <vector>

#include <algorithm> 
#include <functional> 
#include <cctype>
#include <locale>

class StringHelper
{
public:
	template<typename T> static std::vector<std::basic_string<T> > Split(const std::basic_string<T>& input, T delimiter, int count = 0);
	template<typename T> static std::basic_string<T> Replace(const std::basic_string<T>& input, const std::basic_string<T>& search, const std::basic_string<T>& replace);

	static std::wstring ToWString(const char* input);
	static std::string ToAString(const wchar_t* input);

	static std::wstring ToWString(const std::string& input);
	static std::wstring ToWStringNoException(const std::string& input);
	static std::string ToAString(const std::wstring& input);

	template<typename T> static bool StartsWith(const std::basic_string<T>& input, const std::basic_string<T>& what)
	{
		return what.length() <= input.length() && equal(what.begin(), what.end(), input.begin());
	}

	template<typename T> static inline std::basic_string<T> TrimLeft(const std::basic_string<T> &input)
	{
		auto s = input;
		s.erase(s.begin(), std::find_if(s.begin(), s.end(), std::not1(std::ptr_fun<int, int>(std::isspace))));
		return s;
	}

	template<typename T> static inline std::basic_string<T> TrimRight(const std::basic_string<T> &input)
	{
		auto s = input;
		s.erase(std::find_if(s.rbegin(), s.rend(), std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
		return s;
	}

	template<typename T> static inline std::basic_string<T> Trim(const std::basic_string<T> &input)
	{
		return TrimLeft(TrimRight(input));
	}

	template<typename ... TArgs> static void DebugPrint(const std::wstring& format, TArgs ... args)
	{
		auto text = format + L"\n";
		::OutputDebugString(Format(text, args...).c_str());
	}

	template<typename ... TArgs> static std::wstring Format(const std::wstring& format, TArgs ... args)
	{
		std::wstring finalString = format;
		FormatString(finalString, 0, args...);
		return finalString;
	}

private:
	template<typename THead, typename ... TArgs> static void FormatString(std::wstring& format, int index, THead head, TArgs ... args)
	{
		format = StringHelper::Replace(format, L"{" + std::to_wstring(index) + L"}", InternalAdd(head));
		FormatString(format, index + 1, args...);
	}

	template<typename THead, typename ... TArgs> static void FormatString(std::wstring& format, int index, THead head)
	{
		format = StringHelper::Replace(format, L"{" + std::to_wstring(index) + L"}", InternalAdd(head));
	}

	static void FormatString(std::wstring& format, int index)
	{
	}

	template<typename T> static std::wstring InternalAdd(const T& value)
	{
		std::wstringstream ss;
		ss << value;
		return ss.str();
	}

	template<> static std::wstring InternalAdd(const std::string& value)
	{
		return StringHelper::ToWStringNoException(value);
	}

	//template<> static std::wstring InternalAdd(const CefString& value)
	//{
	//	return (std::wstring)value;
	//}
};


template<typename T> std::vector<std::basic_string<T> > StringHelper::Split(const std::basic_string<T>& input, T delimiter, int count)
{
	std::vector<std::basic_string<T> > elems;
	std::basic_stringstream<T> stream(input);
	std::basic_string<T> item;
	int cnt = 0;
	while (std::getline(stream, item, delimiter))
	{
		if (count == 0 || cnt != count)
		{
			elems.push_back(item);
			cnt++;
		}
		else
			elems[elems.size() - 1] = elems[elems.size() - 1] + delimiter + item;
	}
	return elems;
}

template<typename T> static std::basic_string<T> StringHelper::Replace(const std::basic_string<T>& input, const std::basic_string<T>& search, const std::basic_string<T>& replace)
{
	std::basic_string<T> result = input;
	if (input.empty())
		return result;
	
	size_t start_pos = 0;
	while ((start_pos = result.find(search, start_pos)) != std::basic_string<T>::npos)
	{
		result.replace(start_pos, search.length(), replace);
		start_pos += replace.length();
	}

	return result;
}