#pragma once

#include "Component.h"

#include "include/cef_app.h"

#include <string>
#include <mutex>

enum class CheckMustCancelRequestFlags
{
	None = 0x0,
	CheckUrl = 0x1,
	CheckScheme = 0x2,
	CheckContent = 0x4,
	CheckSchemeAndUrl = 0x1 | 0x2,
	CheckAll = 0x1 | 0x2 | 0x4,
};

class EventComponent : public Component
{
public:
	EventComponent(View& view);

	void OnBrowserCreated(CefRefPtr<CefBrowser> browser);

	void OnLoadStart(CefRefPtr<CefFrame> frame);
	void OnLoadEnd(CefRefPtr<CefFrame> frame, int httpStatusCode);
	void OnLoadError(CefRefPtr<CefFrame> frame, int errorCode, const std::wstring& errorText, const std::wstring& failedUrl);
	void OnLoadingStateChange(bool isLoading, bool canGoBack, bool canGoForward);

	bool OnCheckMustCancelRequest(CefRefPtr<CefFrame> frame, const std::wstring& url, const std::wstring& content, CheckMustCancelRequestFlags checkFlags);
	bool OnFileUploadRequest(CefRefPtr<CefFrame> frame, const std::wstring& url, const std::vector<std::wstring>& content);
	bool OnBeforeNavigation(const std::wstring& url, bool isMainFrame, int frameId);
	bool OnBeforeBrowse(const std::wstring& url, bool isMainFrame, bool isRedirect, int frameId);
	void OnContextCreated(int64 frameId);
	void OnContextReleased(int64 frameId);
	void OnTitleChanged(const std::wstring& title);
	void OnGotFocus(bool isForegroundWindow);
	void OnEditableNodeFocusChangedHandler(bool isEditable, int64 frameId);
	void OnFaviconURLChange(const std::vector<std::wstring>& iconUrls);
	void OnPopupViewCreated(int popupViewId, HANDLE popupWindowHandle);

	void OnViewIsReady();
	void OnRendererCreated();

	void OnActivated();
	void OnDeactivated();

private:
	int64 editableFrame;
	std::recursive_mutex mutex;
};

