#pragma once

#include "Component.h"

#include "include/cef_app.h"

#include <string>
#include <vector>
#include <memory>
#include <functional>
#include <mutex>

class Cancelable;

struct AuthenticationResult
{
	bool ShouldCancel;
	std::wstring Username;
	std::wstring Password;
};


struct DownloadRequestResult
{
	bool ShouldCancel;
	std::wstring Destination;
};


struct DisplayDialogResult
{
	bool Success;
	std::wstring UserInput;
};


struct DisplayFileDialogResult
{
	bool Success;
	int SelectedAcceptFilter;
	std::vector<std::wstring> FilePaths;
};


class RequestComponent : public Component
{
public:
	RequestComponent(View& view);
	~RequestComponent();

	void SetContentFilterLevel(int level);
	int GetContentFilterLevel() const;

	void RequestAuthCredentials(bool isProxy, const std::wstring& host, int port, const std::wstring& realm, const std::wstring& scheme, const std::function<void(const AuthenticationResult&)>& callback);
	void RequestQuota(const std::wstring& origin, int64 newSize, const std::function<void(bool)>& callback);
	void RequestDownload(const std::wstring& url, int64 totalBytes, const std::function<void(const DownloadRequestResult&)>& callback);
	void DisplayDialog(const std::wstring& origin, const std::wstring& acceptLang, int dialogType, const std::wstring& message, const std::wstring& defaultPromptText, const std::function<void(const DisplayDialogResult&)>& callback);
	void DisplayFileDialog(int mode, const std::wstring& title, const std::wstring& default_file_path, const std::vector<std::wstring>& acceptFilters, int selectedAcceptFilter, const std::function<void(const DisplayFileDialogResult&)>& callback);

	void CancelRequests();

private:
	std::recursive_mutex mutex;
	int contentFilterLevel;

	std::shared_ptr<Cancelable> quotaRequestCancelable;
	std::shared_ptr<Cancelable> authRequestCancelable;
	std::shared_ptr<Cancelable> downloadRequestCancelable;
	std::shared_ptr<Cancelable> displayDialogCancelable;
	std::shared_ptr<Cancelable> displayFileDialogCancelable;
};

