#pragma once

#include "View.h"
#include "../Thread/ManualResetEvent.h"
#include "include/cef_app.h"

#include <vector>
#include <memory>
#include <mutex>

class ViewManager
{
public:
	ViewManager();
	~ViewManager();

	std::shared_ptr<View> Create();
	std::shared_ptr<View> Create(const CreateViewParams& params);
	void CreateFromPopup(std::shared_ptr<View> creator, CefWindowInfo& windowInfo, CefRefPtr<CefBrowser> browser, CefBrowserSettings& settings);
	
	void CloseView(int viewId, const std::function<void()>& closed);
	void RemoveView(std::shared_ptr<View> view);

	std::shared_ptr<View> GetView(CefRefPtr<CefBrowser> browser);
	std::shared_ptr<View> GetView(int id);
	std::shared_ptr<View> GetView(HWND hwnd);
	std::shared_ptr<View> GetRootView(int id);
	std::shared_ptr<View> GetRootView();

	ManualResetEvent& GetViewExistingEvent();
	void Ping();

	void OnRendererCreated(int browserId);

	bool HasViews();
	void Shutdown(std::function<void()> callback);

	void SetLastFocusedView(int viewId);
	void OnZOrderChanged(int viewId);
	void OnViewDestructed(int viewId);

private:
	std::map<int, std::vector<std::function<void()>>> registeredViewClosedEvents;

	int lastFocusedView;
	ManualResetEvent viewsExisting;
	std::recursive_mutex mutex;
	std::function<void()> shutDownCallback;

	std::vector<std::shared_ptr<View>> views;
	
	int nextViewId;
	bool isPinging;
	int receivedPongResponses;
};

